@@ -1,6 +1,5 @@
-/*
-   Copyright (C) 2002-2004, 2006 MySQL AB & Ramil Kalimullin
-    All rights reserved. Use is subject to license terms.
+/* Copyright (c) 2002-2007 MySQL AB
+   
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include "myisamdef.h"
 
