@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -151,6 +151,7 @@ public class NdbRecordOperationImpl implements Operation {
         this.ndbRecordValues = clusterTransaction.getCachedNdbRecordImpl(storeTable);
         this.valueBufferSize = ndbRecordValues.getBufferSize();
         this.valueBuffer = ndbRecordValues.newBuffer();
+        this.storeColumns = ndbRecordValues.storeColumns;
         this.numberOfColumns = ndbRecordValues.getNumberOfColumns();
         this.blobs = new NdbRecordBlobImpl[this.numberOfColumns];
         resetMask();
@@ -173,7 +174,13 @@ public class NdbRecordOperationImpl implements Operation {
         this.storeColumns = ndbRecordOperationImpl2.ndbRecordValues.storeColumns;
         this.numberOfColumns = this.storeColumns.length;
         this.blobs = new NdbRecordBlobImpl[this.numberOfColumns];
-        this.activeBlobs = ndbRecordOperationImpl2.activeBlobs;
+        // copy the blob data from the original but leave out the NdbBlob part
+        // which is bound to the NdbOperation and is not suitable
+        for (int i = 0; i < ndbRecordOperationImpl2.blobs.length; ++i) {
+            if (ndbRecordOperationImpl2.blobs[i] != null) {
+              this.blobs[i] = new NdbRecordBlobImpl(this, ndbRecordOperationImpl2.blobs[i]);
+            }
+        }
         resetMask();
     }
 
@@ -207,12 +214,18 @@ public class NdbRecordOperationImpl implements Operation {
         valueBuffer.position(0);
         ndbOperation = clusterTransactionImpl.updateTuple(ndbRecordValues.getNdbRecord(), valueBuffer, mask, null);
         clusterTransactionImpl.addOperationToCheck(this);
-        // for each blob column set, get the blob handle and write the values
-        for (NdbRecordBlobImpl blob: activeBlobs) {
-            // activate the blob by getting the NdbBlob
-            blob.setNdbBlob();
-            // set values into the blob
-            blob.setValue();
+        // calculate the active blobs
+        activeBlobs.clear();
+        for (NdbRecordBlobImpl blob: blobs) {
+            if (blob != null && isColumnSet(blob.getColumnId())) {
+                activeBlobs.add(blob);
+                blob.setNdbBlob();
+                blob.setValue();
+            }
+        }
+        if (!activeBlobs.isEmpty()) {
+            // if any blobs, execute the transaction to enable the blob values to be updated
+            clusterTransactionImpl.executeNoCommit();
         }
         return;
     }
@@ -325,9 +338,15 @@ public class NdbRecordOperationImpl implements Operation {
      * @return the blob handle
      */
     public Blob getBlobHandle(Column storeColumn) {
+        if (logger.isDetailEnabled()) {
+            logger.detail("column: " + storeColumn.getName());
+        }
         int columnId = storeColumn.getColumnId();
         NdbRecordBlobImpl result = blobs[columnId];
         if (result == null) {
+            if (logger.isDetailEnabled()) {
+                logger.detail("column: " + storeColumn.getName() + " was null; activating.");
+            }
             columnSet(columnId);
             result = new NdbRecordBlobImpl(this, storeColumn);
             blobs[columnId] = result;
@@ -396,6 +415,10 @@ public class NdbRecordOperationImpl implements Operation {
     }
 
     public void setBytes(Column storeColumn, byte[] value) {
+        if (logger.isDetailEnabled()) {
+            logger.detail("NdbRecordOperationImpl.setBytes for: " + storeColumn.getName() + " value: " + 
+                    Utility.dumpBytes(value));
+        }
         if (value == null) {
             setNull(storeColumn);
         } else {
@@ -580,9 +603,24 @@ public class NdbRecordOperationImpl implements Operation {
     }
 
     public NdbBlob getNdbBlob(Column storeColumn) {
-        NdbBlob result = ndbOperation.getBlobHandle(storeColumn.getColumnId());
-        handleError(result, ndbOperation);
+        if (ndbOperation == null) {
+            throw new ClusterJFatalInternalException("NdbRecordOperationImpl.getNdbBlob with no ndbOperation.");
+        } else {
+            NdbBlob result = ndbOperation.getBlobHandle(storeColumn.getColumnId());
+            handleError(result, ndbOperation);
         return result;
+        }
+    }
+
+    /** Activate blob/text columns just read
+     * 
+     */
+    public void activateBlobColumns() {
+        for (Column storeColumn: storeColumns) {
+            if (storeColumn.isLob()) {
+                getBlobHandle(storeColumn);
+            }
+        }
     }
 
     /**
@@ -593,7 +631,13 @@ public class NdbRecordOperationImpl implements Operation {
         int byteOffset = columnId / 8;
         int bitInByte = columnId - (byteOffset * 8);
         mask[byteOffset] |= NdbRecordImpl.BIT_IN_BYTE_MASK[bitInByte];
-        
+    }
+
+    protected boolean isColumnSet(int columnId) {
+        int byteOffset = columnId / 8;
+        int bitInByte = columnId - (byteOffset * 8);
+        byte testMask = NdbRecordImpl.BIT_IN_BYTE_MASK[bitInByte];
+        return (mask[byteOffset] & testMask) == testMask;
     }
 
     public NdbRecordImpl getValueNdbRecord() {
