@@ -1,6 +1,5 @@
-/*
-   Copyright (c) 2004-2006, 2008 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (c) 2004-2006 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,6 +22,7 @@
 
 #define CFG_ACC_FRAGMENT      (PRIVATE_BASE +  3)
 #define CFG_ACC_OP_RECS       (PRIVATE_BASE +  4)
+/* TODO: remove. CFG_ACC_OVERFLOW_RECS obsoleted ... */
 #define CFG_ACC_OVERFLOW_RECS (PRIVATE_BASE +  5)
 #define CFG_ACC_PAGE8         (PRIVATE_BASE +  6)
 #define CFG_ACC_ROOT_FRAG     (PRIVATE_BASE +  7)
@@ -62,4 +62,6 @@
 #define CFG_TUX_ATTRIBUTE     (PRIVATE_BASE + 42)
 #define CFG_TUX_SCAN_OP       (PRIVATE_BASE + 43)
 
+#define CFG_SPJ_TABLE         (PRIVATE_BASE + 44)
+
 #endif
