@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +12,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef CNTR_MASTERCONF_HPP
 #define CNTR_MASTERCONF_HPP
 
 #include <NodeBitmask.hpp>
 
+#define JAM_FILE_ID 154
+
+
 /**
  * This signals is sent by NdbCntr-Master to NdbCntr
  */
@@ -46,4 +48,7 @@ private:
   Uint32 theNodes[NdbNodeBitmask::Size];
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
