@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2008 MySQL AB, 2008 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +12,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef LQH_TRANS_CONF_H
 #define LQH_TRANS_CONF_H
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 21
+
+
 /**
  * This signal is sent as response to a LQH_TRANSREQ
  * which is sent as by a take-over TC
@@ -226,4 +228,7 @@ LqhTransConf::setMarkerFlag(UintR & requestInfo, UintR val){
   requestInfo |= (val << LTC_MARKER_SHIFT);
 }
 
+
+#undef JAM_FILE_ID
+
 #endif
