@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef TC_INDX_H
 #define TC_INDX_H
@@ -22,6 +22,12 @@
 #include "TcKeyReq.hpp"
 #include "TcKeyConf.hpp"
 
+#define JAM_FILE_ID 211
+
+
 typedef TcKeyConf TcIndxConf;
 
+
+#undef JAM_FILE_ID
+
 #endif
