@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,14 +12,17 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef TCKEYREF_HPP
 #define TCKEYREF_HPP
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 177
+
+
 class TcKeyRef {
 
   /**
@@ -53,4 +56,7 @@ private:
   Uint32 errorData;
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
