@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef TC_KEY_REQ_H
 #define TC_KEY_REQ_H
@@ -21,6 +21,9 @@
 #include "SignalData.hpp"
 #include <transporter/TransporterDefinitions.hpp>
 
+#define JAM_FILE_ID 127
+
+
 /**
  * @class TcKeyReq
  * @brief Contains KeyInfo and AttrInfo and is commonly followed by more signals
@@ -115,14 +118,12 @@ private:
    *   It includes all attribute info sent in possible attrinfo 
    *   signals as well as the attribute info sent in TCKEYREQ.
    *
-   *   (APIVERSION << 16 | ATTRINFOLEN)
-   *
    * Long TCKEYREQ
-   *   (APIVERSION << 16)
+   *   ATTRIBUTE INFO (attrinfo) LENGTH is unused in signal.
    *   Get AttrInfoLength from length of section 1, if present.
    *
    */
-  UintR attrLen;              // DATA 2   (also stores API Version)
+  UintR attrLen;              // DATA 2
   UintR tableId;              // DATA 3
   UintR requestInfo;          // DATA 4   Various transaction flags
   UintR tableSchemaVersion;   // DATA 5
@@ -149,9 +150,7 @@ private:
    * Get:ers for attrLen
    */ 
 
-  static Uint16  getAPIVersion(const UintR & attrLen);
   static Uint16  getAttrinfoLen(const UintR & attrLen);
-  static void setAPIVersion(UintR & attrLen, Uint16 apiVersion);
   static void setAttrinfoLen(UintR & attrLen, Uint16 aiLen);
 
 
@@ -317,7 +316,9 @@ private:
  * Attr Len
  *
  n = Attrinfo length(words)   - 16 Bits -> max 65535 (Short TCKEYREQ only)
- a = API version no           - 16 Bits -> max 65535
+ a = removed was API version no  - 16 Bits -> max 65535
+ API version no is more than 16 bits, was not used in kernel
+ (removed in 7.3.3, 7.2.14, 7.1.29, 7.0.40, 6.3.53)
 
            1111111111222222222233
  01234567890123456789012345678901
@@ -325,9 +326,6 @@ private:
  aaaaaaaaaaaaaaaa                   (Long TCKEYREQ)
 */
 
-#define API_VER_NO_SHIFT     (16)
-#define API_VER_NO_MASK      (65535)
-
 #define ATTRLEN_SHIFT        (0)
 #define ATTRLEN_MASK         (65535)
 
@@ -574,19 +572,6 @@ TcKeyReq::setTakeOverScanInfo(UintR & scanInfo, Uint32 aScanInfo){
 }
 
 
-inline
-Uint16
-TcKeyReq::getAPIVersion(const UintR & anAttrLen){
-  return (Uint16)((anAttrLen >> API_VER_NO_SHIFT) & API_VER_NO_MASK);
-}
-
-inline
-void
-TcKeyReq::setAPIVersion(UintR & anAttrLen, Uint16 apiVersion){
-// ASSERT_MAX(apiVersion, API_VER_NO_MASK, "TcKeyReq::setAPIVersion");
-  anAttrLen |= (apiVersion << API_VER_NO_SHIFT);
-}
-
 inline
 Uint16
 TcKeyReq::getAttrinfoLen(const UintR & anAttrLen){
@@ -654,4 +639,7 @@ TcKeyReq::getDeferredConstraints(const UintR & requestInfo){
 }
 
 
+
+#undef JAM_FILE_ID
+
 #endif
