@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2008 MySQL AB, 2008-2010 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,6 +35,8 @@ class NdbBlob;
 class TcKeyReq;
 class NdbRecord;
 class NdbInterpretedCode;
+struct GenericSectionPtr;
+class NdbLockHandle;
 
 /**
  * @class NdbOperation
@@ -228,7 +229,7 @@ public:
    * dirtyRead is a deprecated name for committedRead
    *
    * @return 0 if successful otherwise -1.
-   * @depricated
+   * @deprecated
    */
   virtual int			dirtyRead();
 
@@ -805,6 +806,12 @@ public:
    * @note A method also exists without the error parameter.
    * 
    * @param ErrorCode   An error code given by the application programmer.
+   *                    If not supplied, defaults to 899. Applications should 
+   *                    use error code 626 or any code in the [6000-6999] 
+   *                    range.  Error code 899 is supported for backwards 
+   *                    compatibility, but 626 is recommmended instead. For 
+   *                    other codes, the behavior is undefined and may change 
+   *                    at any time without prior notice.
    * @return            -1 if unsuccessful.
    */
   int   interpret_exit_nok(Uint32 ErrorCode);
@@ -913,6 +920,8 @@ public:
     DeleteRequest = 3,            ///< Delete Operation
     WriteRequest = 4,             ///< Write Operation
     ReadExclusive = 5,            ///< Read exclusive
+    RefreshRequest = 6,           ///<
+    UnlockRequest = 7,            ///< Unlock operation
     OpenScanRequest,              ///< Scan Operation
     OpenRangeScanRequest,         ///< Range scan operation
     NotDefined2,                  ///< Internal for debugging
@@ -1025,6 +1034,14 @@ public:
    */
   struct OperationOptions
   {
+    /*
+      Size of the OperationOptions structure.
+    */
+    static inline Uint32 size()
+    {
+        return sizeof(OperationOptions);
+    }
+
     /*
      * Which options are present.  See below for option details
      */
@@ -1035,7 +1052,12 @@ public:
                  OO_PARTITION_ID = 0x08, 
                  OO_INTERPRETED  = 0x10,
                  OO_ANYVALUE     = 0x20,
-                 OO_CUSTOMDATA   = 0x40 };
+                 OO_CUSTOMDATA   = 0x40,
+                 OO_LOCKHANDLE   = 0x80,
+                 OO_QUEUABLE     = 0x100,
+                 OO_NOT_QUEUABLE = 0x200,
+                 OO_DEFERRED_CONSTAINTS = 0x400
+    };
 
     /* An operation-specific abort option.
      * Only necessary if the default abortoption behaviour
@@ -1066,6 +1088,16 @@ public:
     void * customData;
   };
 
+  /* getLockHandle
+   * Returns a pointer to this operation's LockHandle.
+   * For NdbRecord, the lock handle must first be requested using
+   * the OO_LOCKHANDLE operation option.
+   * For non-NdbRecord operations, this call can be used alone.
+   * The returned LockHandle cannot be used until the operation
+   * has been executed.
+   */
+  const NdbLockHandle* getLockHandle() const;
+  const NdbLockHandle* getLockHandle();
 
 protected:
 /******************************************************************************
@@ -1185,6 +1217,9 @@ protected:
   virtual void	    release();			// Release all operations 
                                                 // connected to
 					      	// the operations object.      
+  void              postExecuteRelease();       // Release resources
+                                                // no longer needed after
+                                                // exceute 
   void		    setStartIndicator();
 
   /* Utility method to 'add' operation options to an NdbOperation
@@ -1205,6 +1240,7 @@ protected:
  * was sent, then the connection object is told about this situation.
  *****************************************************************************/
 
+  int    doSendKeyReq(int processorId, GenericSectionPtr* secs, Uint32 numSecs);
   int    doSend(int ProcessorId, Uint32 lastFlag);
   virtual int	 prepareSend(Uint32  TC_ConnectPtr,
                              Uint64  TransactionId,
@@ -1235,7 +1271,8 @@ protected:
                                      Uint32 &wordsWritten);
 
   // Method which prepares signals at operation definition time.
-  int    buildSignalsNdbRecord(Uint32 aTC_ConnectPtr, Uint64 aTransId);
+  int    buildSignalsNdbRecord(Uint32 aTC_ConnectPtr, Uint64 aTransId,
+                               const Uint32 * read_mask);
 
   // Method which does final preparations at execute time.
   int    prepareSendNdbRecord(AbortOption ao);
@@ -1244,34 +1281,20 @@ protected:
   Uint32 fillTcKeyReqHdr(TcKeyReq *tcKeyReq,
                          Uint32 connectPtr,
                          Uint64 transId);
-  int    allocKeyInfo(Uint32 connectPtr, Uint64 transId,
-                      Uint32 **dstPtr, Uint32 *remain);
-  int    allocAttrInfo(Uint32 connectPtr, Uint64 transId,
-                       Uint32 **dstPtr, Uint32 *remain);
-  int    insertKEYINFO_NdbRecord(Uint32 connectPtr,
-                                 Uint64 transId,
-                                 const char *value,
-                                 Uint32 size,
-                                 Uint32 **dstPtr,
-                                 Uint32 *remain);
-  int    insertATTRINFOHdr_NdbRecord(Uint32 connectPtr,
-                                     Uint64 transId,
-                                     Uint32 attrId,
-                                     Uint32 attrLen,
-                                     Uint32 **dstPtr,
-                                     Uint32 *remain);
-  int    insertATTRINFOData_NdbRecord(Uint32 connectPtr,
-                                      Uint64 transId,
-                                      const char *value,
-                                      Uint32 size,
-                                      Uint32 **dstPtr,
-                                      Uint32 *remain);
+  int    allocKeyInfo();
+  int    allocAttrInfo();
+  int    insertKEYINFO_NdbRecord(const char *value,
+                                 Uint32 byteSize);
+  int    insertATTRINFOHdr_NdbRecord(Uint32 attrId,
+                                     Uint32 attrLen);
+  int    insertATTRINFOData_NdbRecord(const char *value,
+                                      Uint32 size);
 
-  int	 receiveTCKEYREF(NdbApiSignal*); 
+  int	 receiveTCKEYREF(const NdbApiSignal*);
 
   int	 checkMagicNumber(bool b = true); // Verify correct object
 
-  int    checkState_TransId(NdbApiSignal* aSignal);
+  int    checkState_TransId(const NdbApiSignal* aSignal);
 
 /******************************************************************************
  *	These are support methods only used locally in this class.
@@ -1295,8 +1318,9 @@ protected:
   NdbBlob *linkInBlobHandle(NdbTransaction *aCon,
                             const NdbColumnImpl *column,
                             NdbBlob * & lastPtr);
-  int getBlobHandlesNdbRecord(NdbTransaction* aCon);
-  int getBlobHandlesNdbRecordDelete(NdbTransaction* aCon, bool checkReadSet);
+  int getBlobHandlesNdbRecord(NdbTransaction* aCon, const Uint32 * mask);
+  int getBlobHandlesNdbRecordDelete(NdbTransaction* aCon, 
+                                    bool checkReadSet, const Uint32 * mask);
 
   // Handle ATTRINFO signals   
   int insertATTRINFO(Uint32 aData);
@@ -1321,10 +1345,15 @@ protected:
   int	      insertBranch(Uint32 aBranch);
 
   Uint32 ptr2int() { return theReceiver.getId(); };
+  Uint32 ptr2int() const { return theReceiver.getId(); };
 
   // get table or index key from prepared signals
   int getKeyFromTCREQ(Uint32* data, Uint32 & size);
 
+  int getLockHandleImpl();
+  int prepareGetLockHandle();
+  int prepareGetLockHandleNdbRecord();
+
   virtual void setReadLockMode(LockMode lockMode);
 
 /******************************************************************************
@@ -1345,6 +1374,7 @@ protected:
   union {
     NdbApiSignal* theTCREQ;		// The TC[KEY/INDX]REQ signal object
     NdbApiSignal* theSCAN_TABREQ;
+    NdbApiSignal* theRequest;
   };
 
   NdbApiSignal*	   theFirstATTRINFO;	// The first ATTRINFO signal object 
@@ -1367,7 +1397,9 @@ protected:
   Uint32	    theNoOfSubroutines;
 
   Uint32*           theKEYINFOptr;       // Pointer to where to write KEYINFO
+  Uint32            keyInfoRemain;       // KeyInfo space in current signal
   Uint32*           theATTRINFOptr;      // Pointer to where to write ATTRINFO
+  Uint32            attrInfoRemain;      // AttrInfo space in current signal
 
   /* 
      The table object for the table to read or modify (for index operations,
@@ -1414,12 +1446,21 @@ protected:
                                  // Note that scan operations always have this
                                  // set true
   Int8  theDistrKeyIndicator_;    // Indicates whether distr. key is used
-  Uint8  m_no_disk_flag;          
-  /*
-    For NdbRecord, this flag indicates that we need to send the Event-attached
-    word set by setAnyValue().
-  */
-  Uint8 m_use_any_value;
+
+  enum OP_FLAGS {
+    OF_NO_DISK = 0x1,
+
+    /*
+      For NdbRecord, this flag indicates that we need to send the Event-attached
+      word set by setAnyValue().
+    */
+    OF_USE_ANY_VALUE = 0x2,
+    OF_QUEUEABLE = 0x4,
+    OF_DEFERRED_CONSTRAINTS = 0x8
+  };
+  Uint8  m_flags;
+
+  Uint8 _unused1;
 
   Uint16 m_tcReqGSN;
   Uint16 m_keyInfoGSN;
@@ -1456,7 +1497,7 @@ protected:
     Do not use clas Bitmask/BitmaskPOD here, to avoid having to
     #include <Bitmask.hpp> in application code.
   */
-  Uint32 m_read_mask[(NDB_MAX_ATTRIBUTES_IN_TABLE+31)>>5];
+  Uint32 m_unused_read_mask[(128+31)>>5];
   /* Interpreted program for NdbRecord operations. */
   const NdbInterpretedCode *m_interpreted_code;
 
@@ -1492,6 +1533,12 @@ protected:
 
   Uint32 repack_read(Uint32 len);
 
+  NdbLockHandle* theLockHandle;
+
+  bool m_blob_lock_upgraded; /* Did Blob code upgrade LM_CommittedRead
+                              * to LM_Read?
+                              */
+
 private:
   NdbOperation(const NdbOperation&); // Not impl.
   NdbOperation&operator=(const NdbOperation&);
