@@ -1,6 +1,5 @@
-/*
-   Copyright (c) 2003-2006, 2008 MySQL AB, 2008, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef NDBOUT_H
 #define NDBOUT_H
@@ -76,7 +75,8 @@ public:
   NdbOut& endline(void);
   NdbOut& flushline(void);
   NdbOut& setHexFormat(int _format);
-  
+  NdbOut& hexdump(const Uint32 * words, size_t count);  
+
   NdbOut();
   NdbOut(OutputStream &, bool autoflush = true);
   virtual ~NdbOut();
@@ -93,11 +93,6 @@ private:
   bool m_autoflush;
 };
 
-#ifdef NDB_WIN
-typedef int(*NdbOutF)(char*);
-extern NdbOutF ndbout_svc;
-#endif
-
 inline NdbOut& NdbOut::operator<<(NdbOut& (* _f)(NdbOut&)) {
   (* _f)(*this); 
   return * this; 
