@@ -1,6 +1,5 @@
-/*
-   Copyright (c) 2003-2008 MySQL AB, 2008 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (c) 2003-2005, 2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,16 +12,14 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include <ndb_global.h> 
 
 #include "SCI_Transporter.hpp" 
 #include <NdbOut.hpp> 
 #include <NdbSleep.h> 
-#include <NdbTick.h> 
-#include <NdbTick.h> 
 
 #include "TransporterInternalDefinitions.hpp" 
 #include <TransporterCallback.hpp> 
@@ -315,10 +312,6 @@ sci_error_t SCI_Transporter::initLocalSegment() {
  
  
 bool SCI_Transporter::doSend() { 
-#ifdef DEBUG_TRANSPORTER  
-  NDB_TICKS startSec=0, stopSec=0; 
-  Uint32 startMicro=0, stopMicro=0, totalMicro=0; 
-#endif
   sci_error_t             err; 
   Uint32 retry=0; 
  
