@@ -1,6 +1,5 @@
-/*
-   Copyright (c) 2004-2007 MySQL AB
-   Use is subject to license terms.
+/* Copyright (c) 2004-2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -60,16 +59,35 @@ getTypeOf(Uint32 k) {
   return (ConfigValues::ValueType)((k >> KP_TYPE_SHIFT) & KP_TYPE_MASK);
 }
 
-ConfigValues::ConfigValues(Uint32 sz, Uint32 dsz){
-  m_size = sz;
-  m_dataSize = dsz;
+ConfigValues::ConfigValues(Uint32 sz, Uint32 dsz)
+: m_size(sz),
+  m_dataSize(dsz)
+{
+  assert(dsz % sizeof(char const*) == 0);
+  assert(dsz % sizeof(Uint64) == 0);
+  assert(my_offsetof(ConfigValues, m_values) % 8 == 0);
   m_stringCount = 0;
   m_int64Count = 0;
   for(Uint32 i = 0; i<m_size; i++){
-    m_values[i << 1] = CFV_KEY_FREE;
+   m_values[i << 1] = CFV_KEY_FREE;
   }
 }
 
+ConfigValues*
+ConfigValues::constructInPlace(Uint32 keys, Uint32 data, void* place, size_t size)
+{
+  assert(size >= sizeInBytes(keys, data));
+  return new (place) ConfigValues(keys, data);
+}
+
+size_t
+ConfigValues::sizeInBytes(Uint32 keys, Uint32 data)
+{
+  return my_offsetof(ConfigValues, m_values) +
+         keys * 2 * sizeof(Uint32) +
+         data * sizeof(char);
+}
+
 ConfigValues::~ConfigValues(){
   for(Uint32 i = 0; i<m_stringCount; i++){
     free(* getString(i));
@@ -129,12 +147,14 @@ ConfigValues::getString(Uint32 index) const {
   const Uint32 * data = m_values + (m_size << 1);
   char * ptr = (char*)data;
   ptr += m_dataSize; 
-  ptr -= (index * sizeof(char *));
+  ptr -= ((index + 1) * sizeof(char *));
   return (char**)ptr;
 }
 
 bool
 ConfigValues::ConstIterator::openSection(Uint32 key, Uint32 no){
+  // Nested section are not in use. See also ConfigValuesFactory::openSection().
+  assert(m_currentSection == 0);
   Uint32 curr = m_currentSection;
   
   Entry tmp;
@@ -310,17 +330,18 @@ ConfigValuesFactory::ConfigValuesFactory(ConfigValues * cfg){
 ConfigValuesFactory::~ConfigValuesFactory()
 {
   if(m_cfg)
+  {
+    m_cfg->~ConfigValues();
     free(m_cfg);
+  }
 }
 
 ConfigValues *
-ConfigValuesFactory::create(Uint32 keys, Uint32 data){
-  Uint32 sz = sizeof(ConfigValues);
-  sz += (2 * keys * sizeof(Uint32)); 
-  sz += data;
-  
+ConfigValuesFactory::create(Uint32 keys, Uint32 data)
+{
+  size_t sz = ConfigValues::sizeInBytes(keys, data);
   void * tmp = malloc(sz);
-  return new (tmp) ConfigValues(keys, data);
+  return ConfigValues::constructInPlace(keys, data, tmp, sz);
 }
 
 void
@@ -363,6 +384,13 @@ ConfigValuesFactory::shrink(){
 
 bool
 ConfigValuesFactory::openSection(Uint32 key, Uint32 no){
+  /*
+    NOTE: The code seems to suggest that sections could be nested, so that
+    you could open a section and then call openSection() again to open
+    a subsection. But this pattern is currently not in use. The assert below 
+    prevents accidential use of nested sections.
+  */
+  assert(m_currentSection == 0);
   ConfigValues::Entry tmp;
   const Uint32 parent = m_currentSection;
 
@@ -475,6 +503,7 @@ ConfigValuesFactory::put(const ConfigValues::Entry & entry){
     }
   }
   
+  assert(m_freeKeys > 0);
   switch(entry.m_type){
   case ConfigValues::IntType:
   case ConfigValues::SectionType:
@@ -491,6 +520,7 @@ ConfigValuesFactory::put(const ConfigValues::Entry & entry){
     char **  ref = m_cfg->getString(index);
     * ref = strdup(entry.m_string ? entry.m_string : "");
     m_freeKeys--;
+    assert(m_freeData >= sizeof(char*));
     m_freeData -= sizeof(char *);
     DEBUG printf("Putting at: %d(%d) (loop = %d) key: %d value(%d): %s\n", 
 		   pos, sz, 0, 
@@ -504,6 +534,7 @@ ConfigValuesFactory::put(const ConfigValues::Entry & entry){
     m_cfg->m_values[pos+1] = index;
     * m_cfg->get64(index) = entry.m_int64;
     m_freeKeys--;
+    assert(m_freeData >= 8);
     m_freeData -= 8;
     DEBUG printf("Putting at: %d(%d) (loop = %d) key: %d value64(%d): %lld\n", 
 		   pos, sz, 0, 
@@ -563,7 +594,7 @@ ConfigValuesFactory::extractCurrentSection(const ConfigValues::ConstIterator & c
 ConfigValues *
 ConfigValuesFactory::getConfigValues(){
   ConfigValues * ret = m_cfg;
-  m_cfg = create(10, 10);
+  m_cfg = create(10, 16);
   return ret;
 }
 
@@ -577,8 +608,9 @@ Uint32
 ConfigValues::getPackedSize() const {
 
   Uint32 size = 0;
+  Uint32 const* values = m_values;
   for(Uint32 i = 0; i < 2 * m_size; i += 2){
-    Uint32 key = m_values[i];
+    Uint32 key = values[i];
     if(key != CFV_KEY_FREE){
       switch(::getTypeOf(key)){
       case IntType:
@@ -590,7 +622,7 @@ ConfigValues::getPackedSize() const {
 	break;
       case StringType:
 	size += 8; // key + len
-	size += mod4(strlen(* getString(m_values[i+1])) + 1);
+        size += mod4((unsigned)strlen(* getString(values[i+1])) + 1);
 	break;
       case InvalidType:
       default:
@@ -608,9 +640,10 @@ ConfigValues::pack(void * _dst, Uint32 _len) const {
   char * dst = (char*)_dst;
   memcpy(dst, Magic, sizeof(Magic)); dst += sizeof(Magic);
 
+  Uint32 const* values = m_values;
   for(i = 0; i < 2 * m_size; i += 2){
-    Uint32 key = m_values[i];
-    Uint32 val = m_values[i+1];
+    Uint32 key = values[i];
+    Uint32 val = values[i+1];
     if(key != CFV_KEY_FREE){
       switch(::getTypeOf(key)){
       case IntType:
@@ -620,8 +653,8 @@ ConfigValues::pack(void * _dst, Uint32 _len) const {
 	break;
       case Int64Type:{
 	Uint64 i64 = * get64(val); 
-	Uint32 hi = (i64 >> 32);
-	Uint32 lo = (i64 & 0xFFFFFFFF);
+	Uint32 hi = (Uint32)(i64 >> 32);
+	Uint32 lo = (Uint32)(i64 & 0xFFFFFFFF);
 	* (Uint32*)dst = htonl(key); dst += 4;
 	* (Uint32*)dst = htonl(hi); dst += 4;
 	* (Uint32*)dst = htonl(lo); dst += 4;
@@ -629,7 +662,7 @@ ConfigValues::pack(void * _dst, Uint32 _len) const {
 	break;
       case StringType:{
 	const char * str = * getString(val);
-	Uint32 len = strlen(str) + 1;
+	Uint32 len = Uint32(strlen(str) + 1);
 	* (Uint32*)dst = htonl(key); dst += 4;
 	* (Uint32*)dst = htonl(len); dst += 4;
 	memcpy(dst, str, len); 
@@ -645,7 +678,7 @@ ConfigValues::pack(void * _dst, Uint32 _len) const {
   }
 
   const Uint32 * sum = (Uint32*)_dst;
-  const Uint32 len = ((Uint32*)dst) - sum;
+  const Uint32 len = Uint32(((Uint32*)dst) - sum);
   Uint32 chk = 0;
   for(i = 0; i<len; i++){
     chk ^= htonl(sum[i]);
