@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef DBDIH_H
 #define DBDIH_H
@@ -30,6 +30,10 @@
 #include <signaldata/LCP.hpp>
 #include <NdbSeqLock.hpp>
 #include <CountingSemaphore.hpp>
+#include <Mutex.hpp>
+
+#define JAM_FILE_ID 356
+
 
 #ifdef DBDIH_C
 
@@ -589,7 +593,9 @@ public:
   /* WE KEEP IT IN A RECORD TO ENABLE IT TO BE PARALLELISED IN THE FUTURE.  */
   /**************************************************************************/
   struct TakeOverRecord {
-    
+
+    TakeOverRecord() {}
+
     /**
      * States possible on slave (starting node)
      */
@@ -642,7 +648,7 @@ public:
     Uint32 toCurrentTabref;
     Uint32 toFailedNode;
     Uint32 toStartingNode;
-    Uint64 toStartTime;
+    NDB_TICKS toStartTime;
     ToSlaveStatus toSlaveStatus;
     ToMasterStatus toMasterStatus;
    
@@ -1397,7 +1403,7 @@ private:
       State m_state;
       Uint32 m_new_gci;
       Uint32 m_time_between_gcp;   /* Delay between global checkpoints */
-      Uint64 m_start_time;
+      NDB_TICKS m_start_time;
     } m_master;
   } m_gcp_save;
 
@@ -1430,7 +1436,7 @@ private:
       State m_state;
       Uint32 m_time_between_gcp;
       Uint64 m_new_gci;
-      Uint64 m_start_time;
+      NDB_TICKS m_start_time;
     } m_master;
   } m_micro_gcp;
 
@@ -1439,16 +1445,18 @@ private:
     struct
     {
       Uint32 m_gci;
-      Uint32 m_counter;
-      Uint32 m_max_lag;
+      Uint32 m_elapsed_ms; //MilliSec since last GCP_SAVEed
+      Uint32 m_max_lag_ms; //Max allowed lag(ms) before 'crashSystem'
     } m_gcp_save;
 
     struct
     {
       Uint64 m_gci;
-      Uint32 m_counter;
-      Uint32 m_max_lag;
+      Uint32 m_elapsed_ms; //MilliSec since last GCP_COMMITed
+      Uint32 m_max_lag_ms; //Max allowed lag(ms) before 'crashSystem'
     } m_micro_gcp;
+
+    NDB_TICKS m_last_check; //Time GCP monitor last checked
   } m_gcp_monitor;
 
   /*------------------------------------------------------------------------*/
@@ -1546,9 +1554,9 @@ private:
     Uint32 keepGci;      /* USED TO CALCULATE THE GCI TO KEEP AFTER A LCP  */
     Uint32 oldestRestorableGci;
     
-    Uint64 m_start_time; // When last LCP was started
-    Uint64 m_lcp_time;   // How long last LCP took
-    Uint32 m_lcp_trylock_timeout;
+    NDB_TICKS m_start_time; // When last LCP was started
+    Uint64    m_lcp_time;   // How long last LCP took
+    Uint32    m_lcp_trylock_timeout;
 
     struct CurrentFragment {
       Uint32 tableId;
@@ -1598,6 +1606,7 @@ private:
                                     THIS NUMBER IS INCREMENTED. AT THE START OF
                                     THE SYSTEM THIS NUMBER MUST BE INITIATED TO
                                     ZERO */
+  Uint32 cMinTcFailNo;            /* Minimum TC handled failNo allowed to close GCP */
 
   BlockReference clocallqhblockref;
   BlockReference clocaltcblockref;
@@ -1612,7 +1621,7 @@ private:
   Uint32 csystemnodes;
   Uint32 c_newest_restorable_gci;
   Uint32 c_set_initial_start_flag;
-  Uint64 c_current_time; // Updated approx. every 10ms
+  NDB_TICKS c_current_time; // Updated approx. every 10ms
 
   /* Limit the number of concurrent table definition writes during LCP
    * This avoids exhausting the DIH page pool
@@ -1924,11 +1933,21 @@ private:
   // MT LQH
   Uint32 c_fragments_per_node_;
   Uint32 getFragmentsPerNode();
+  /**
+   * dihGetInstanceKey
+   *
+   * This method maps a fragment to a block instance key
+   * This is the LDM instance which manages the fragment
+   * on this node.
+   * The range of an instance key is 1 to 
+   * NDBMT_MAX_WORKER_INSTANCES inclusive.
+   * 0 is the proxy block instance.
+   */
   Uint32 dihGetInstanceKey(FragmentstorePtr tFragPtr) {
     ndbrequire(!tFragPtr.isNull());
     Uint32 log_part_id = tFragPtr.p->m_log_part_id;
-    Uint32 instanceKey = 1 + (log_part_id % NDBMT_MAX_BLOCK_INSTANCES);
-    return instanceKey;
+    ndbrequire(log_part_id < NDBMT_MAX_WORKER_INSTANCES);
+    return 1 + log_part_id;
   }
   Uint32 dihGetInstanceKey(Uint32 tabId, Uint32 fragId);
 
@@ -1944,5 +1963,8 @@ private:
 #error "cdata is to small compared to Sysfile size"
 #endif
 
+
+#undef JAM_FILE_ID
+
 #endif
 
