@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,6 +27,9 @@
 #include <signaldata/DropIndxImpl.hpp>
 #include <signaldata/AbortAll.hpp>
 
+#define JAM_FILE_ID 348
+
+
 
 class DbtcProxy : public DbgdmProxy {
 public:
@@ -90,6 +93,7 @@ protected:
   // GSN_GCP_NOMORETRANS
   struct Ss_GCP_NOMORETRANS : SsParallel {
     GCPNoMoreTrans m_req;
+    Uint32 m_minTcFailNo;
     Ss_GCP_NOMORETRANS() {
       m_sendREQ = (SsFUNCREQ)&DbtcProxy::sendGCP_NOMORETRANS;
       m_sendCONF = (SsFUNCREP)&DbtcProxy::sendGCP_TCFINISHED;
@@ -186,4 +190,7 @@ protected:
   void sendABORT_ALL_CONF(Signal*, Uint32 ssId);
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
