@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #define DBTUP_C
 #define DBTUP_COMMIT_CPP
@@ -24,6 +24,9 @@
 #include <signaldata/TupCommit.hpp>
 #include "../dblqh/Dblqh.hpp"
 
+#define JAM_FILE_ID 416
+
+
 void Dbtup::execTUP_DEALLOCREQ(Signal* signal)
 {
   TablerecPtr regTabPtr;
@@ -42,7 +45,7 @@ void Dbtup::execTUP_DEALLOCREQ(Signal* signal)
   getFragmentrec(regFragPtr, frag_id, regTabPtr.p);
   ndbassert(regFragPtr.p != NULL);
   
-  if (! (((frag_page_id << MAX_TUPLES_BITS) + page_index) == ~ (Uint32) 0))
+  if (! Local_key::isInvalid(frag_page_id, page_index))
   {
     Local_key tmp;
     tmp.m_page_no= getRealpid(regFragPtr.p, frag_page_id); 
@@ -51,15 +54,8 @@ void Dbtup::execTUP_DEALLOCREQ(Signal* signal)
     PagePtr pagePtr;
     Tuple_header* ptr= (Tuple_header*)get_ptr(&pagePtr, &tmp, regTabPtr.p);
 
-    ndbassert(ptr->m_header_bits & Tuple_header::FREE);
+    ndbassert(ptr->m_header_bits & Tuple_header::FREED);
 
-    if (ptr->m_header_bits & Tuple_header::LCP_KEEP)
-    {
-      ndbassert(! (ptr->m_header_bits & Tuple_header::FREED));
-      ptr->m_header_bits |= Tuple_header::FREED;
-      return;
-    }
-    
     if (regTabPtr.p->m_attributes[MM].m_no_of_varsize +
         regTabPtr.p->m_attributes[MM].m_no_of_dynamic)
     {
@@ -109,7 +105,6 @@ void Dbtup::initOpConnection(Operationrec* regOperPtr)
 {
   set_tuple_state(regOperPtr, TUPLE_ALREADY_ABORTED);
   set_trans_state(regOperPtr, TRANS_IDLE);
-  regOperPtr->currentAttrinbufLen= 0;
   regOperPtr->op_struct.op_type= ZREAD;
   regOperPtr->op_struct.m_disk_preallocated= 0;
   regOperPtr->op_struct.m_load_diskpage_on_commit= 0;
@@ -155,15 +150,16 @@ Dbtup::is_rowid_lcp_scanned(const Local_key& key1,
 
 void
 Dbtup::dealloc_tuple(Signal* signal,
-		     Uint32 gci,
+		     Uint32 gci_hi,
+                     Uint32 gci_lo,
 		     Page* page,
 		     Tuple_header* ptr, 
+                     KeyReqStruct * req_struct,
 		     Operationrec* regOperPtr, 
 		     Fragrecord* regFragPtr, 
 		     Tablerec* regTabPtr)
 {
   Uint32 lcpScan_ptr_i= regFragPtr->m_lcp_scan_op;
-  Uint32 lcp_keep_list = regFragPtr->m_lcp_keep_list;
 
   Uint32 bits = ptr->m_header_bits;
   Uint32 extra_bits = Tuple_header::FREED;
@@ -173,10 +169,10 @@ Dbtup::dealloc_tuple(Signal* signal,
     Local_key disk;
     memcpy(&disk, ptr->get_disk_ref_ptr(regTabPtr), sizeof(disk));
     PagePtr tmpptr;
-    tmpptr.i = m_pgman.m_ptr.i;
-    tmpptr.p = reinterpret_cast<Page*>(m_pgman.m_ptr.p);
+    tmpptr.i = m_pgman_ptr.i;
+    tmpptr.p = reinterpret_cast<Page*>(m_pgman_ptr.p);
     disk_page_free(signal, regTabPtr, regFragPtr, 
-		   &disk, tmpptr, gci);
+		   &disk, tmpptr, gci_hi);
   }
   
   if (! (bits & (Tuple_header::LCP_SKIP | Tuple_header::ALLOC)) && 
@@ -190,9 +186,15 @@ Dbtup::dealloc_tuple(Signal* signal,
     if (!is_rowid_lcp_scanned(rowid, *scanOp.p))
     {
       jam();
-      extra_bits = Tuple_header::LCP_KEEP; // Note REMOVE FREE
-      ptr->m_operation_ptr_i = lcp_keep_list;
-      regFragPtr->m_lcp_keep_list = rowid.ref();
+
+      /**
+       * We're committing a delete, on a row that should
+       *   be part of LCP. Copy original row into copy-tuple
+       *   and add this copy-tuple to lcp-keep-list
+       *
+       */
+      handle_lcp_keep_commit(&rowid,
+                             req_struct, regOperPtr, regFragPtr, regTabPtr);
     }
   }
   
@@ -201,10 +203,78 @@ Dbtup::dealloc_tuple(Signal* signal,
   if (regTabPtr->m_bits & Tablerec::TR_RowGCI)
   {
     jam();
-    * ptr->get_mm_gci(regTabPtr) = gci;
+    * ptr->get_mm_gci(regTabPtr) = gci_hi;
+    if (regTabPtr->m_bits & Tablerec::TR_ExtraRowGCIBits)
+    {
+      Uint32 attrId = regTabPtr->getExtraAttrId<Tablerec::TR_ExtraRowGCIBits>();
+      store_extra_row_bits(attrId, regTabPtr, ptr, gci_lo, /* truncate */true);
+    }
   }
 }
 
+void
+Dbtup::handle_lcp_keep_commit(const Local_key* rowid,
+                              KeyReqStruct * req_struct,
+                              Operationrec * opPtrP,
+                              Fragrecord * regFragPtr,
+                              Tablerec * regTabPtr)
+{
+  bool disk = false;
+  Uint32 sizes[4];
+  Uint32 * copytuple = get_copy_tuple_raw(&opPtrP->m_copy_tuple_location);
+  Tuple_header * dst = get_copy_tuple(copytuple);
+  Tuple_header * org = req_struct->m_tuple_ptr;
+  if (regTabPtr->need_expand(disk))
+  {
+    setup_fixed_part(req_struct, opPtrP, regTabPtr);
+    req_struct->m_tuple_ptr = dst;
+    expand_tuple(req_struct, sizes, org, regTabPtr, disk);
+    shrink_tuple(req_struct, sizes+2, regTabPtr, disk);
+  }
+  else
+  {
+    memcpy(dst, org, 4*regTabPtr->m_offsets[MM].m_fix_header_size);
+  }
+  dst->m_header_bits |= Tuple_header::COPY_TUPLE;
+
+  /**
+   * Store original row-id in copytuple[0,1]
+   * Store next-ptr in copytuple[1,2] (set to RNIL/RNIL)
+   *
+   */
+  assert(sizeof(Local_key) == 8);
+  memcpy(copytuple+0, rowid, sizeof(Local_key));
+
+  Local_key nil;
+  nil.setNull();
+  memcpy(copytuple+2, &nil, sizeof(nil));
+
+  /**
+   * Link it to list
+   */
+  if (regFragPtr->m_lcp_keep_list_tail.isNull())
+  {
+    jam();
+    regFragPtr->m_lcp_keep_list_head = opPtrP->m_copy_tuple_location;
+  }
+  else
+  {
+    jam();
+    Uint32 * tail = get_copy_tuple_raw(&regFragPtr->m_lcp_keep_list_tail);
+    Local_key nextptr;
+    memcpy(&nextptr, tail+2, sizeof(Local_key));
+    ndbassert(nextptr.isNull());
+    nextptr = opPtrP->m_copy_tuple_location;
+    memcpy(tail+2, &nextptr, sizeof(Local_key));
+  }
+  regFragPtr->m_lcp_keep_list_tail = opPtrP->m_copy_tuple_location;
+
+  /**
+   * And finally clear m_copy_tuple_location so that it won't be freed
+   */
+  opPtrP->m_copy_tuple_location.setNull();
+}
+
 #if 0
 static void dump_buf_hex(unsigned char *p, Uint32 bytes)
 {
@@ -227,7 +297,8 @@ static void dump_buf_hex(unsigned char *p, Uint32 bytes)
 
 void
 Dbtup::commit_operation(Signal* signal,
-			Uint32 gci,
+			Uint32 gci_hi,
+                        Uint32 gci_lo,
 			Tuple_header* tuple_ptr, 
 			PagePtr pagePtr,
 			Operationrec* regOperPtr, 
@@ -248,6 +319,7 @@ Dbtup::commit_operation(Signal* signal,
   Uint32 fixsize= regTabPtr->m_offsets[MM].m_fix_header_size;
   Uint32 mm_vars= regTabPtr->m_attributes[MM].m_no_of_varsize;
   Uint32 mm_dyns= regTabPtr->m_attributes[MM].m_no_of_dynamic;
+  bool update_gci_at_commit = ! regOperPtr->op_struct.m_gci_written;
   if((mm_vars+mm_dyns) == 0)
   {
     jam();
@@ -331,14 +403,14 @@ Dbtup::commit_operation(Signal* signal,
     memcpy(&key, copy->get_disk_ref_ptr(regTabPtr), sizeof(Local_key));
     Uint32 logfile_group_id= regFragPtr->m_logfile_group_id;
 
-    PagePtr diskPagePtr = { (Tup_page*)m_pgman.m_ptr.p, m_pgman.m_ptr.i };
+    PagePtr diskPagePtr = { (Tup_page*)m_pgman_ptr.p, m_pgman_ptr.i };
     ndbassert(diskPagePtr.p->m_page_no == key.m_page_no);
     ndbassert(diskPagePtr.p->m_file_no == key.m_file_no);
     Uint32 sz, *dst;
     if(copy_bits & Tuple_header::DISK_ALLOC)
     {
       jam();
-      disk_page_alloc(signal, regTabPtr, regFragPtr, &key, diskPagePtr, gci);
+      disk_page_alloc(signal, regTabPtr, regFragPtr, &key, diskPagePtr, gci_hi);
     }
     
     if(regTabPtr->m_attributes[DD].m_no_of_varsize == 0)
@@ -358,7 +430,7 @@ Dbtup::commit_operation(Signal* signal,
     {
       jam();
       disk_page_undo_update(diskPagePtr.p, 
-			    &key, dst, sz, gci, logfile_group_id);
+			    &key, dst, sz, gci_hi, logfile_group_id);
     }
     
     memcpy(dst, disk_ptr, 4*sz);
@@ -391,10 +463,17 @@ Dbtup::commit_operation(Signal* signal,
   tuple_ptr->m_header_bits= copy_bits;
   tuple_ptr->m_operation_ptr_i= save;
   
-  if (regTabPtr->m_bits & Tablerec::TR_RowGCI)
+  if (regTabPtr->m_bits & Tablerec::TR_RowGCI  &&
+      update_gci_at_commit)
   {
     jam();
-    * tuple_ptr->get_mm_gci(regTabPtr) = gci;
+    * tuple_ptr->get_mm_gci(regTabPtr) = gci_hi;
+    if (regTabPtr->m_bits & Tablerec::TR_ExtraRowGCIBits)
+    {
+      Uint32 attrId = regTabPtr->getExtraAttrId<Tablerec::TR_ExtraRowGCIBits>();
+      store_extra_row_bits(attrId, regTabPtr, tuple_ptr, gci_lo,
+                           /* truncate */true);
+    }
   }
   
   if (regTabPtr->m_bits & Tablerec::TR_Checksum) {
@@ -409,12 +488,14 @@ Dbtup::disk_page_commit_callback(Signal* signal,
 {
   Uint32 hash_value;
   Uint32 gci_hi, gci_lo;
+  Uint32 transId1, transId2;
   OperationrecPtr regOperPtr;
 
   jamEntry();
   
   c_operation_pool.getPtr(regOperPtr, opPtrI);
-  c_lqh->get_op_info(regOperPtr.p->userpointer, &hash_value, &gci_hi, &gci_lo);
+  c_lqh->get_op_info(regOperPtr.p->userpointer, &hash_value, &gci_hi, &gci_lo,
+                     &transId1, &transId2);
 
   TupCommitReq * const tupCommitReq= (TupCommitReq *)signal->getDataPtr();
   
@@ -423,15 +504,17 @@ Dbtup::disk_page_commit_callback(Signal* signal,
   tupCommitReq->gci_hi= gci_hi;
   tupCommitReq->gci_lo= gci_lo;
   tupCommitReq->diskpage = page_id;
+  tupCommitReq->transId1 = transId1;
+  tupCommitReq->transId2 = transId2;
 
   regOperPtr.p->op_struct.m_load_diskpage_on_commit= 0;
   regOperPtr.p->m_commit_disk_callback_page= page_id;
-  m_global_page_pool.getPtr(m_pgman.m_ptr, page_id);
+  m_global_page_pool.getPtr(m_pgman_ptr, page_id);
   
   {
     PagePtr tmp;
-    tmp.i = m_pgman.m_ptr.i;
-    tmp.p = reinterpret_cast<Page*>(m_pgman.m_ptr.p);
+    tmp.i = m_pgman_ptr.i;
+    tmp.p = reinterpret_cast<Page*>(m_pgman_ptr.p);
     disk_page_set_dirty(tmp);
   }
   
@@ -450,12 +533,14 @@ Dbtup::disk_page_log_buffer_callback(Signal* signal,
 {
   Uint32 hash_value;
   Uint32 gci_hi, gci_lo;
+  Uint32 transId1, transId2;
   OperationrecPtr regOperPtr;
 
   jamEntry();
   
   c_operation_pool.getPtr(regOperPtr, opPtrI);
-  c_lqh->get_op_info(regOperPtr.p->userpointer, &hash_value, &gci_hi, &gci_lo);
+  c_lqh->get_op_info(regOperPtr.p->userpointer, &hash_value, &gci_hi, &gci_lo,
+                     &transId1, &transId2);
   Uint32 page= regOperPtr.p->m_commit_disk_callback_page;
 
   TupCommitReq * const tupCommitReq= (TupCommitReq *)signal->getDataPtr();
@@ -465,10 +550,12 @@ Dbtup::disk_page_log_buffer_callback(Signal* signal,
   tupCommitReq->gci_hi= gci_hi;
   tupCommitReq->gci_lo= gci_lo;
   tupCommitReq->diskpage = page;
+  tupCommitReq->transId1 = transId1;
+  tupCommitReq->transId2 = transId2;
 
   ndbassert(regOperPtr.p->op_struct.m_load_diskpage_on_commit == 0);
   regOperPtr.p->op_struct.m_wait_log_buffer= 0;
-  m_global_page_pool.getPtr(m_pgman.m_ptr, page);
+  m_global_page_pool.getPtr(m_pgman_ptr, page);
   
   execTUP_COMMITREQ(signal);
   ndbassert(signal->theData[0] == 0);
@@ -476,6 +563,86 @@ Dbtup::disk_page_log_buffer_callback(Signal* signal,
   c_lqh->tupcommit_conf_callback(signal, regOperPtr.p->userpointer);
 }
 
+int Dbtup::retrieve_data_page(Signal *signal,
+                              Page_cache_client::Request req,
+                              OperationrecPtr regOperPtr)
+{
+  req.m_callback.m_callbackData= regOperPtr.i;
+  req.m_callback.m_callbackFunction =
+    safe_cast(&Dbtup::disk_page_commit_callback);
+
+  /*
+   * Consider commit to be correlated.  Otherwise pk op + commit makes
+   * the page hot.   XXX move to TUP which knows better.
+   */
+  int flags= regOperPtr.p->op_struct.op_type |
+    Page_cache_client::COMMIT_REQ | Page_cache_client::CORR_REQ;
+  Page_cache_client pgman(this, c_pgman);
+  int res= pgman.get_page(signal, req, flags);
+  m_pgman_ptr = pgman.m_ptr;
+
+  switch(res){
+  case 0:
+    /**
+     * Timeslice
+     */
+    jam();
+    signal->theData[0] = 1;
+    return res;
+  case -1:
+    ndbrequire("NOT YET IMPLEMENTED" == 0);
+    break;
+  default:
+    jam();
+  }
+  {
+    PagePtr tmpptr;
+    tmpptr.i = m_pgman_ptr.i;
+    tmpptr.p = reinterpret_cast<Page*>(m_pgman_ptr.p);
+
+    disk_page_set_dirty(tmpptr);
+  }
+  regOperPtr.p->m_commit_disk_callback_page= res;
+  regOperPtr.p->op_struct.m_load_diskpage_on_commit= 0;
+
+  return res;
+}
+
+int Dbtup::retrieve_log_page(Signal *signal,
+                             FragrecordPtr regFragPtr,
+                             OperationrecPtr regOperPtr)
+{
+  jam();
+  /**
+   * Only last op on tuple needs "real" commit,
+   *   hence only this one should have m_wait_log_buffer
+   */
+
+  CallbackPtr cb;
+  cb.m_callbackData= regOperPtr.i;
+  cb.m_callbackIndex = DISK_PAGE_LOG_BUFFER_CALLBACK;
+  Uint32 sz= regOperPtr.p->m_undo_buffer_space;
+
+  D("Logfile_client - execTUP_COMMITREQ");
+  Logfile_client lgman(this, c_lgman, regFragPtr.p->m_logfile_group_id);
+  int res= lgman.get_log_buffer(signal, sz, &cb);
+  jamEntry();
+  switch(res){
+  case 0:
+    jam();
+    signal->theData[0] = 1;
+    return res;
+  case -1:
+    ndbrequire("NOT YET IMPLEMENTED" == 0);
+    break;
+  default:
+    jam();
+  }
+  regOperPtr.p->op_struct.m_wait_log_buffer= 0;
+
+  return res;
+}
+
 /**
  * Move to the first operation performed on this tuple
  */
@@ -501,7 +668,7 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
   FragrecordPtr regFragPtr;
   OperationrecPtr regOperPtr;
   TablerecPtr regTabPtr;
-  KeyReqStruct req_struct;
+  KeyReqStruct req_struct(this, KRS_COMMIT);
   TransState trans_state;
   Uint32 no_of_fragrec, no_of_tablerec;
 
@@ -511,6 +678,8 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
   Uint32 hash_value= tupCommitReq->hashValue;
   Uint32 gci_hi = tupCommitReq->gci_hi;
   Uint32 gci_lo = tupCommitReq->gci_lo;
+  Uint32 transId1 = tupCommitReq->transId1;
+  Uint32 transId2 = tupCommitReq->transId2;
 
   jamEntry();
 
@@ -531,13 +700,17 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
   req_struct.hash_value= hash_value;
   req_struct.gci_hi = gci_hi;
   req_struct.gci_lo = gci_lo;
+  /* Put transid in req_struct, so detached triggers can access it */
+  req_struct.trans_id1 = transId1;
+  req_struct.trans_id2 = transId2;
+  req_struct.m_reorg = regOperPtr.p->op_struct.m_reorg;
   regOperPtr.p->m_commit_disk_callback_page = tupCommitReq->diskpage;
 
 #ifdef VM_TRACE
   if (tupCommitReq->diskpage == RNIL)
   {
-    m_pgman.m_ptr.i = RNIL;
-    m_pgman.m_ptr.p = 0;
+    m_pgman_ptr.i = RNIL;
+    m_pgman_ptr.p = 0;
     req_struct.m_disk_page_ptr.i = RNIL;
     req_struct.m_disk_page_ptr.p = 0;
   }
@@ -622,8 +795,9 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
         disk_page_abort_prealloc(signal, regFragPtr.p, 
 				 &req.m_page, req.m_page.m_page_idx);
         
-        c_lgman->free_log_space(regFragPtr.p->m_logfile_group_id, 
-				regOperPtr.p->m_undo_buffer_space);
+        D("Logfile_client - execTUP_COMMITREQ");
+        Logfile_client lgman(this, c_lgman, regFragPtr.p->m_logfile_group_id);
+        lgman.free_log_space(regOperPtr.p->m_undo_buffer_space);
 	goto skip_disk;
         if (0) ndbout_c("insert+delete");
         jamEntry();
@@ -640,42 +814,12 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
       
       ndbassert(tuple_ptr->m_header_bits & Tuple_header::DISK_PART);
     }
-    req.m_callback.m_callbackData= regOperPtr.i;
-    req.m_callback.m_callbackFunction = 
-      safe_cast(&Dbtup::disk_page_commit_callback);
 
-    /*
-     * Consider commit to be correlated.  Otherwise pk op + commit makes
-     * the page hot.   XXX move to TUP which knows better.
-     */
-    int flags= regOperPtr.p->op_struct.op_type |
-      Page_cache_client::COMMIT_REQ | Page_cache_client::CORR_REQ;
-    int res= m_pgman.get_page(signal, req, flags);
-    switch(res){
-    case 0:
-      /**
-       * Timeslice
-       */
-      jam();
-      signal->theData[0] = 1;
-      return;
-    case -1:
-      ndbrequire("NOT YET IMPLEMENTED" == 0);
-      break;
-    default:
-      jam();
+    if (retrieve_data_page(signal, req, regOperPtr) == 0)
+    {
+      return; // Data page has not been retrieved yet.
     }
     get_page = true;
-
-    {
-      PagePtr tmpptr;
-      tmpptr.i = m_pgman.m_ptr.i;
-      tmpptr.p = reinterpret_cast<Page*>(m_pgman.m_ptr.p);
-      disk_page_set_dirty(tmpptr);
-    }
-    
-    regOperPtr.p->m_commit_disk_callback_page= res;
-    regOperPtr.p->op_struct.m_load_diskpage_on_commit= 0;
   } 
   
   if(regOperPtr.p->op_struct.m_wait_log_buffer)
@@ -687,25 +831,9 @@ void Dbtup::execTUP_COMMITREQ(Signal* signal)
      */
     ndbassert(tuple_ptr->m_operation_ptr_i == regOperPtr.i);
     
-    Callback cb;
-    cb.m_callbackData= regOperPtr.i;
-    cb.m_callbackFunction = 
-      safe_cast(&Dbtup::disk_page_log_buffer_callback);
-    Uint32 sz= regOperPtr.p->m_undo_buffer_space;
-    
-    Logfile_client lgman(this, c_lgman, regFragPtr.p->m_logfile_group_id);
-    int res= lgman.get_log_buffer(signal, sz, &cb);
-    jamEntry();
-    switch(res){
-    case 0:
-      jam();
-      signal->theData[0] = 1;
-      return;
-    case -1:
-      ndbrequire("NOT YET IMPLEMENTED" == 0);
-      break;
-    default:
-      jam();
+    if (retrieve_log_page(signal, regFragPtr, regOperPtr) == 0)
+    {
+      return; // Log page has not been retrieved yet.
     }
   }
   
@@ -738,23 +866,29 @@ skip_disk:
     
     tuple_ptr->m_operation_ptr_i = RNIL;
     
-    if(regOperPtr.p->op_struct.op_type != ZDELETE)
+    if (regOperPtr.p->op_struct.op_type == ZDELETE)
     {
       jam();
-      commit_operation(signal, gci_hi, tuple_ptr, page,
+      if (get_page)
+      {
+        ndbassert(tuple_ptr->m_header_bits & Tuple_header::DISK_PART);
+      }
+      dealloc_tuple(signal, gci_hi, gci_lo, page.p, tuple_ptr,
+                    &req_struct, regOperPtr.p, regFragPtr.p, regTabPtr.p);
+    }
+    else if(regOperPtr.p->op_struct.op_type != ZREFRESH)
+    {
+      jam();
+      commit_operation(signal, gci_hi, gci_lo, tuple_ptr, page,
 		       regOperPtr.p, regFragPtr.p, regTabPtr.p); 
     }
     else
     {
       jam();
-      if (get_page)
-      {
-	ndbassert(tuple_ptr->m_header_bits & Tuple_header::DISK_PART);
-      }
-      dealloc_tuple(signal, gci_hi, page.p, tuple_ptr,
-		    regOperPtr.p, regFragPtr.p, regTabPtr.p); 
+      commit_refresh(signal, gci_hi, gci_lo, tuple_ptr, page,
+                     &req_struct, regOperPtr.p, regFragPtr.p, regTabPtr.p);
     }
-  } 
+  }
 
   if (nextOp != RNIL)
   {
@@ -790,18 +924,64 @@ Dbtup::set_commit_change_mask_info(const Tablerec* regTabPtr,
   else
   {
     Uint32 * dst = req_struct->changeMask.rep.data;
-    Uint32 * maskptr = get_copy_tuple_raw(&regOperPtr->m_copy_tuple_location);
-    Uint32 cols = * maskptr;
+    Uint32 * rawptr = get_copy_tuple_raw(&regOperPtr->m_copy_tuple_location);
+    ChangeMask * maskptr = get_change_mask_ptr(rawptr);
+    Uint32 cols = maskptr->m_cols;
     if (cols == regTabPtr->m_no_of_attributes)
     {
-      memcpy(dst, maskptr + 1, 4*masklen);
+      memcpy(dst, maskptr->m_mask, 4*masklen);
     }
     else
     {
       ndbassert(regTabPtr->m_no_of_attributes > cols); // no drop column
-      memcpy(dst, maskptr + 1, 4*((cols + 31) >> 5));
+      memcpy(dst, maskptr->m_mask, 4*((cols + 31) >> 5));
       req_struct->changeMask.setRange(cols,
                                       regTabPtr->m_no_of_attributes - cols);
     }
   }
 }
+
+void
+Dbtup::commit_refresh(Signal* signal,
+                      Uint32 gci_hi,
+                      Uint32 gci_lo,
+                      Tuple_header* tuple_ptr,
+                      PagePtr pagePtr,
+                      KeyReqStruct * req_struct,
+                      Operationrec* regOperPtr,
+                      Fragrecord* regFragPtr,
+                      Tablerec* regTabPtr)
+{
+  /* Committing a refresh operation.
+   * Refresh of an existing row looks like an update
+   * and can commit normally.
+   * Refresh of a non-existing row looks like an Insert which
+   * is 'undone' at commit time.
+   * This is achieved by making special calls to ACC to get
+   * it to forget, before deallocating the tuple locally.
+   */
+  switch(regOperPtr->m_copy_tuple_location.m_file_no){
+  case Operationrec::RF_SINGLE_NOT_EXIST:
+  case Operationrec::RF_MULTI_NOT_EXIST:
+    break;
+  case Operationrec::RF_SINGLE_EXIST:
+  case Operationrec::RF_MULTI_EXIST:
+    // "Normal" update
+    commit_operation(signal, gci_hi, gci_lo, tuple_ptr, pagePtr,
+                     regOperPtr, regFragPtr, regTabPtr);
+    return;
+
+  default:
+    ndbrequire(false);
+  }
+
+  Local_key key = regOperPtr->m_tuple_location;
+  key.m_page_no = pagePtr.p->frag_page_id;
+
+  /**
+   * Tell ACC to delete
+   */
+  c_lqh->accremoverow(signal, regOperPtr->userpointer, &key);
+  dealloc_tuple(signal, gci_hi, gci_lo, pagePtr.p, tuple_ptr,
+                req_struct, regOperPtr, regFragPtr, regTabPtr);
+}
