@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #define DBTUP_C
 #define DBTUP_PAG_MAN_CPP
@@ -23,6 +22,9 @@
 #include <ndb_limits.h>
 #include <pc.hpp>
 
+#define JAM_FILE_ID 407
+
+
 /* ---------------------------------------------------------------- */
 // 4) Page Memory Manager (buddy algorithm)
 //
@@ -58,7 +60,8 @@
 //
 // The following routines are part of the external interface:
 // void
-// allocConsPages(Uint32  noOfPagesToAllocate, #In
+// allocConsPages(EmulatedJamBuffer *jamBuff   #In/out
+//                Uint32  noOfPagesToAllocate, #In
 //                Uint32& noOfPagesAllocated,  #Out
 //                Uint32& retPageRef)          #Out
 // void
@@ -117,29 +120,46 @@ Uint32 Dbtup::nextHigherTwoLog(Uint32 input)
 
 void Dbtup::initializePage() 
 {
-  cnoOfAllocatedPages = 0;
 }//Dbtup::initializePage()
 
-void Dbtup::allocConsPages(Uint32 noOfPagesToAllocate,
+void Dbtup::allocConsPages(EmulatedJamBuffer* jamBuf,
+                           Uint32 noOfPagesToAllocate,
                            Uint32& noOfPagesAllocated,
                            Uint32& allocPageRef)
 {
   if (noOfPagesToAllocate == 0){ 
-    jam();
+    thrjam(jamBuf);
     noOfPagesAllocated = 0;
     return;
   }//if
 
-  m_ctx.m_mm.alloc_pages(RT_DBTUP_PAGE, &allocPageRef, 
+  Resource_limit rl;
+  m_ctx.m_mm.get_resource_limit_nolock(RG_DATAMEM, rl);
+  if (rl.m_curr + m_minFreePages + noOfPagesToAllocate > rl.m_max)
+  {
+    thrjam(jamBuf);
+    noOfPagesAllocated = 0;
+    return;
+  }
+
+  m_ctx.m_mm.alloc_pages(RT_DBTUP_PAGE, &allocPageRef,
 			 &noOfPagesToAllocate, 1);
-  cnoOfAllocatedPages += noOfPagesToAllocate;
   noOfPagesAllocated = noOfPagesToAllocate;
+
+  // Count number of allocated pages
+  m_pages_allocated += noOfPagesAllocated;
+  if (m_pages_allocated > m_pages_allocated_max)
+    m_pages_allocated_max = m_pages_allocated;
+
   return;
 }//allocConsPages()
 
 void Dbtup::returnCommonArea(Uint32 retPageRef, Uint32 retNo) 
 {
   m_ctx.m_mm.release_pages(RT_DBTUP_PAGE, retPageRef, retNo);
-  cnoOfAllocatedPages -= retNo;
+
+  // Count number of allocated pages
+  m_pages_allocated -= retNo;
+
 }//Dbtup::returnCommonArea()
 
