@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2005-2008 MySQL AB, 2010 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include "restore.hpp"
 #include <signaldata/FsRef.hpp>
@@ -29,14 +28,17 @@
 #include <signaldata/LqhKey.hpp>
 #include <AttributeHeader.hpp>
 #include <md5_hash.hpp>
-#include <Dblqh.hpp>
+#include <dblqh/Dblqh.hpp>
 #include <dbtup/Dbtup.hpp>
 #include <KeyDescriptor.hpp>
 
+#define JAM_FILE_ID 453
+
+
 #define PAGES LCP_RESTORE_BUFFER
 
-Restore::Restore(Block_context& ctx) :
-  SimulatedBlock(RESTORE, ctx),
+Restore::Restore(Block_context& ctx, Uint32 instanceNumber) :
+  SimulatedBlock(RESTORE, ctx, instanceNumber),
   m_file_list(m_file_pool),
   m_file_hash(m_file_pool)
 {
@@ -74,8 +76,9 @@ Restore::execSTTOR(Signal* signal)
 {
   jamEntry();                            
 
-  c_lqh = (Dblqh*)globalData.getBlock(DBLQH);
-  c_tup = (Dbtup*)globalData.getBlock(DBTUP);
+  c_lqh = (Dblqh*)globalData.getBlock(DBLQH, instance());
+  c_tup = (Dbtup*)globalData.getBlock(DBTUP, instance());
+  ndbrequire(c_lqh != 0 && c_tup != 0);
   sendSTTORRY(signal);
   
   return;
@@ -193,7 +196,8 @@ Restore::sendSTTORRY(Signal* signal){
   signal->theData[3] = 1;
   signal->theData[4] = 3;
   signal->theData[5] = 255; // No more start phases from missra
-  sendSignal(NDBCNTR_REF, GSN_STTORRY, signal, 6, JBB);
+  BlockReference cntrRef = !isNdbMtLqh() ? NDBCNTR_REF : RESTORE_REF;
+  sendSignal(cntrRef, GSN_STTORRY, signal, 6, JBB);
 }
 
 void
@@ -236,7 +240,7 @@ Restore::execRESTORE_LCP_REQ(Signal* signal){
   do
   {
     FilePtr file_ptr;
-    if(!m_file_list.seize(file_ptr))
+    if (!m_file_list.seizeFirst(file_ptr))
     {
       err= RestoreLcpRef::NoFileRecord;
       break;
@@ -360,6 +364,11 @@ Restore::release_file(FilePtr file_ptr)
 void
 Restore::open_file(Signal* signal, FilePtr file_ptr)
 {
+  signal->theData[0] = NDB_LE_StartReadLCP;
+  signal->theData[1] = file_ptr.p->m_table_id;
+  signal->theData[2] = file_ptr.p->m_fragment_id;
+  sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 3, JBB);
+
   FsOpenReq * req = (FsOpenReq *)signal->getDataPtrSend();
   req->userReference = reference();
   req->fileFlags = FsOpenReq::OM_READONLY | FsOpenReq::OM_GZ;
@@ -430,6 +439,7 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
     Uint32 left= file_ptr.p->m_bytes_left;
     if(left < 8)
     {
+      jam();
       /**
        * Not enought bytes to read header
        */
@@ -442,18 +452,22 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
     Uint32 pos= file_ptr.p->m_current_page_pos;
     if(status & File::READING_RECORDS)
     {
+      jam();
       /**
        * We are reading records
        */
       len= ntohl(* (page_ptr.p->data + pos)) + 1;
+      ndbrequire(len < GLOBAL_PAGE_SIZE_WORDS);
     }
     else
     {
+      jam();
       /**
        * Section length is in 2 word
        */
       if(pos + 1 == GLOBAL_PAGE_SIZE_WORDS)
       {
+        jam();
 	/**
 	 * But that's stored on next page...
 	 *   and since we have atleast 8 bytes left in buffer
@@ -467,18 +481,22 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
       }
       else
       {
+        jam();
 	len= ntohl(* (page_ptr.p->data + pos + 1));
       }
     }
 
     if(file_ptr.p->m_status & File::FIRST_READ)
     {
+      jam();
       len= 3;
       file_ptr.p->m_status &= ~(Uint32)File::FIRST_READ;
     }
     
     if(4 * len > left)
     {
+      jam();
+
       /**
        * Not enought bytes to read "record"
        */
@@ -499,6 +517,7 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
 
     if(pos + len >= GLOBAL_PAGE_SIZE_WORDS)
     {
+      jam();
       /**
        * But it's split over pages
        */
@@ -513,12 +532,73 @@ Restore::restore_next(Signal* signal, FilePtr file_ptr)
       file_ptr.p->m_current_page_pos = (pos + len) - GLOBAL_PAGE_SIZE_WORDS;
       file_ptr.p->m_current_page_index = 
 	(file_ptr.p->m_current_page_index + 1) % page_count;
-      
-      Uint32 first = (GLOBAL_PAGE_SIZE_WORDS - pos);
-      memcpy(page_ptr.p, page_ptr.p->data+pos, 4 * first);
-      memcpy(page_ptr.p->data+first, next_page_ptr.p, 4 * (len - first));
-      data= page_ptr.p->data;
-    } 
+
+      if (len <= GLOBAL_PAGE_SIZE_WORDS)
+      {
+        jam();
+        Uint32 first = (GLOBAL_PAGE_SIZE_WORDS - pos);
+        // wl4391_todo removing valgrind overlap warning for now
+        memmove(page_ptr.p, page_ptr.p->data+pos, 4 * first);
+        memcpy(page_ptr.p->data+first, next_page_ptr.p, 4 * (len - first));
+        data= page_ptr.p->data;
+      }
+      else
+      {
+        jam();
+        /**
+         * A table definition can be larger than one page...
+         * when that happens copy it out to side buffer
+         *
+         * First copy part belonging to page_ptr
+         * Then copy full middle pages (moving forward in page-list)
+         * Last copy last part
+         */
+        Uint32 save = len;
+        assert(len <= NDB_ARRAY_SIZE(m_table_buf));
+        Uint32 * dst = m_table_buf;
+
+        /**
+         * First
+         */
+        Uint32 first = (GLOBAL_PAGE_SIZE_WORDS - pos);
+        memcpy(dst, page_ptr.p->data+pos, 4 * first);
+        len -= first;
+        dst += first;
+
+        /**
+         * Middle
+         */
+        while (len > GLOBAL_PAGE_SIZE_WORDS)
+        {
+          jam();
+          memcpy(dst, next_page_ptr.p, 4 * GLOBAL_PAGE_SIZE_WORDS);
+          len -= GLOBAL_PAGE_SIZE_WORDS;
+          dst += GLOBAL_PAGE_SIZE_WORDS;
+
+          {
+            LocalDataBuffer<15> pages(m_databuffer_pool, file_ptr.p->m_pages);
+            Uint32 next_page = (file_ptr.p->m_current_page_index + 1) % page_count;
+            pages.position(it, next_page % page_count);
+            m_global_page_pool.getPtr(next_page_ptr, * it.data);
+
+            file_ptr.p->m_current_page_ptr_i = next_page_ptr.i;
+            file_ptr.p->m_current_page_index = next_page;
+          }
+        }
+
+        /**
+         * last
+         */
+        memcpy(dst, next_page_ptr.p, 4 * len);
+        file_ptr.p->m_current_page_pos = len;
+
+        /**
+         * Set pointer and len
+         */
+        len = save;
+        data = m_table_buf;
+      }
+    }
     else
     {
       file_ptr.p->m_current_page_pos = pos + len;
@@ -1044,8 +1124,8 @@ Restore::parse_record(Signal* signal, FilePtr file_ptr,
 
     ndbrequire(disk == false); // Not supported...
     ndbrequire(rowid == true);
-    keyLen = keyData - key_start;
-    attrLen = attrData - attr_start;
+    keyLen = Uint32(keyData - key_start);
+    attrLen = Uint32(attrData - attr_start);
     if (desc->noOfKeyAttr != desc->noOfVarKeys)
     {
       reorder_key(desc, key_start, keyLen);
@@ -1211,7 +1291,7 @@ Restore::execLQHKEYCONF(Signal* signal)
 {
   FilePtr file_ptr;
   LqhKeyConf * conf = (LqhKeyConf *)signal->getDataPtr();
-  m_file_pool.getPtr(file_ptr, conf->connectPtr);
+  m_file_pool.getPtr(file_ptr, conf->opPtr);
   
   ndbassert(file_ptr.p->m_outstanding_operations);
   file_ptr.p->m_outstanding_operations--;
@@ -1248,6 +1328,14 @@ Restore::restore_lcp_conf(Signal* signal, FilePtr file_ptr)
                GSN_RESTORE_LCP_CONF, signal, 
                RestoreLcpConf::SignalLength, JBB);
   }
+
+  signal->theData[0] = NDB_LE_ReadLCPComplete;
+  signal->theData[1] = file_ptr.p->m_table_id;
+  signal->theData[2] = file_ptr.p->m_fragment_id;
+  signal->theData[3] = Uint32(file_ptr.p->m_rows_restored >> 32);
+  signal->theData[4] = Uint32(file_ptr.p->m_rows_restored);
+  sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 5, JBB);
+
   release_file(file_ptr);
 }
 
