@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef SUMA_H
 #define SUMA_H
@@ -23,9 +23,7 @@
 
 #include <NodeBitmask.hpp>
 
-#include <SLList.hpp>
-#include <DLList.hpp>
-#include <DLCFifoList.hpp>
+#include <IntrusiveList.hpp>
 #include <KeyTable.hpp>
 #include <DataBuffer.hpp>
 #include <SignalCounter.hpp>
@@ -36,6 +34,9 @@
 #include <signaldata/SumaImpl.hpp>
 #include <ndbapi/NdbDictionary.hpp>
 
+#define JAM_FILE_ID 469
+
+
 class Suma : public SimulatedBlock {
   BLOCK_DEFINES(Suma);
 public:
@@ -77,6 +78,7 @@ public:
   void execSCAN_FRAGREF(Signal* signal);
   void execSCAN_FRAGCONF(Signal* signal);
   void execTRANSID_AI(Signal* signal);
+  void execKEYINFO20(Signal* signal);
   void execSUB_SYNC_CONTINUE_REF(Signal* signal);
   void execSUB_SYNC_CONTINUE_CONF(Signal* signal);
   
@@ -85,39 +87,46 @@ public:
    */
   void execTRIG_ATTRINFO(Signal* signal);
   void execFIRE_TRIG_ORD(Signal* signal);
+  void execFIRE_TRIG_ORD_L(Signal* signal);
   void execSUB_GCP_COMPLETE_REP(Signal* signal);
   
   /**
    * DIH signals
    */
-  void execDI_FCOUNTREF(Signal* signal);
-  void execDI_FCOUNTCONF(Signal* signal);
-  void execDIGETPRIMREF(Signal* signal);
-  void execDIGETPRIMCONF(Signal* signal);
+  void execDIH_SCAN_TAB_REF(Signal* signal);
+  void execDIH_SCAN_TAB_CONF(Signal* signal);
+  void execDIH_SCAN_GET_NODES_REF(Signal* signal);
+  void execDIH_SCAN_GET_NODES_CONF(Signal* signal);
+  void execCHECKNODEGROUPSCONF(Signal *signal);
+  void execGCP_PREPARE(Signal *signal);
 
   /**
    * Trigger administration
    */
-  void execCREATE_TRIG_REF(Signal* signal);
-  void execCREATE_TRIG_CONF(Signal* signal);
-  void execDROP_TRIG_REF(Signal* signal);
-  void execDROP_TRIG_CONF(Signal* signal);
+  void execCREATE_TRIG_IMPL_REF(Signal* signal);
+  void execCREATE_TRIG_IMPL_CONF(Signal* signal);
+  void execDROP_TRIG_IMPL_REF(Signal* signal);
+  void execDROP_TRIG_IMPL_CONF(Signal* signal);
   
   /**
    * continueb
    */
   void execCONTINUEB(Signal* signal);
 
+  void execCREATE_NODEGROUP_IMPL_REQ(Signal*);
+  void execDROP_NODEGROUP_IMPL_REQ(Signal*);
 public:
 
   void suma_ndbrequire(bool v);
 
-  typedef DataBuffer<15> TableList;
-  
+  // wl4391_todo big enough for now
+  // Keep m_fragDesc within 32 bit,
+  // m_dummy is used to pass value.
   union FragmentDescriptor { 
     struct  {
       Uint16 m_fragmentNo;
-      Uint16 m_nodeId;
+      Uint8 m_lqhInstanceKey;
+      Uint8 m_nodeId;
     } m_fragDesc;
     Uint32 m_dummy;
   };
@@ -143,13 +152,13 @@ public:
   };
   typedef Ptr<Subscriber> SubscriberPtr;
 
-  class Table;
-  friend class Table;
+  struct Table;
+  friend struct Table;
   typedef Ptr<Table> TablePtr;
 
   struct SyncRecord {
     SyncRecord(Suma& s, DataBuffer<15>::DataBufferPool & p)
-      : m_tableList(p), suma(s)
+      : suma(s)
 #ifdef ERROR_INSERT
 	, cerrorInsert(s.cerrorInsert)
 #endif
@@ -164,20 +173,24 @@ public:
     Uint32 m_error;
     Uint32 m_requestInfo;
 
-    Uint32 m_currentTable;
-    TableList m_tableList;    // Tables to sync
-    TableList::DataBufferIterator m_tableList_it;
+    Uint32 m_frag_cnt; // only scan this many fragments...
+    Uint32 m_frag_id;  // only scan this specific fragment...
+    Uint32 m_tableId;  // redundant...
+
+    /**
+     * Fragments
+     */
+    Uint32 m_scan_cookie;
+    DataBuffer<15>::Head m_fragments;  // Fragment descriptors
 
     /**
      * Sync data
      */
     Uint32 m_currentFragment;       // Index in tabPtr.p->m_fragments
-    DataBuffer<15>::Head m_attributeList; // Attribute if other than default
-    DataBuffer<15>::Head m_tabList; // tables if other than default
-    
-    Uint32 m_currentTableId;        // Current table
     Uint32 m_currentNoOfAttributes; // No of attributes for current table
-
+    DataBuffer<15>::Head m_attributeList; // Attribute if other than default
+    DataBuffer<15>::Head m_boundInfo;  // For range scan
+    
     void startScan(Signal*);
     void nextScan(Signal*);
     bool getNextFragment(TablePtr * tab, FragmentDescriptor * fd);
@@ -188,6 +201,7 @@ public:
     UintR &cerrorInsert;
 #endif
     BlockNumber number() const { return suma.number(); }
+    EmulatedJamBuffer *jamBuffer() const { return suma.jamBuffer(); }
     void progError(int line, int cause, const char * extra) { 
       suma.progError(line, cause, extra); 
     }
@@ -231,6 +245,7 @@ public:
     Uint32 m_subscriptionId;
     Uint32 m_subscriptionKey;
     Uint32 m_subscriptionType;
+    Uint32 m_schemaTransId;
     Uint16 m_options;
 
     enum Options {
@@ -316,12 +331,6 @@ public:
 
     Uint32 m_error;
     
-    /**
-     * Fragments
-     */
-    Uint32 m_fragCount;
-    DataBuffer<15>::Head m_fragments;  // Fragment descriptors
-    
     Uint32 m_noOfAttributes;
 
     /**
@@ -335,6 +344,9 @@ public:
     bool equal(const Table& rec) const {
       return m_tableId == rec.m_tableId;
     }
+
+    // copy from Subscription
+    Uint32 m_schemaTransId;
   };
 
   /**
@@ -424,13 +436,13 @@ public:
    */
 
   void getNodeGroupMembers(Signal* signal);
-
   void execREAD_CONFIG_REQ(Signal* signal);
 
   void execSTTOR(Signal* signal);
   void sendSTTORRY(Signal*);
   void execNDB_STTOR(Signal* signal);
   void execDUMP_STATE_ORD(Signal* signal);
+  void execDBINFO_SCANREQ(Signal* signal);
   void execREAD_NODESCONF(Signal* signal);
   void execNODE_FAILREP(Signal* signal);
   void execINCL_NODEREQ(Signal* signal);
@@ -496,6 +508,9 @@ public:
   // for LQH transporter overload check
   const NodeBitmask& getSubscriberNodes() const { return c_subscriber_nodes; }
 
+protected:
+  virtual bool getParam(const char * param, Uint32 * retVal);
+
 private:
   /**
    * Variables
@@ -535,7 +550,8 @@ private:
   } c_restart;
 
   Uint32 c_current_seq; // Sequence no on subscription(s)
-  
+  Uint32 c_outstanding_drop_trig_req;
+
   NodeBitmask c_connected_nodes;  // (NODE/API) START REP / (API/NODE) FAIL REQ
   NodeBitmask c_subscriber_nodes; // 
 
@@ -547,7 +563,8 @@ private:
   Uint32 c_nodesInGroup[MAX_REPLICAS];
   NdbNodeBitmask c_nodes_in_nodegroup_mask;  // NodeId's of nodes in nodegroup
 
-  void send_dict_lock_req(Signal* signal);
+  void send_dict_lock_req(Signal* signal, Uint32 state);
+  void send_dict_unlock_ord(Signal* signal, Uint32 state);
   void send_start_me_req(Signal* signal);
   void check_start_handover(Signal* signal);
   void send_handover_req(Signal* signal, Uint32 type);
@@ -556,6 +573,8 @@ private:
   Uint32 get_responsible_node(Uint32 B, const NdbNodeBitmask& mask) const;
   bool check_switchover(Uint32 bucket, Uint64 gci);
 
+  void fix_nodegroup();
+
 public:  
   struct Page_pos
   {
@@ -573,8 +592,14 @@ private:
       ,BUCKET_HANDOVER = 0x2 // On running node
       ,BUCKET_TAKEOVER = 0x4 // On takeing over node
       ,BUCKET_RESEND   = 0x8 // On takeing over node
-      ,BUCKET_SHUTDOWN = 0x10 // Graceful shutdown (shutdown)
-      ,BUCKET_SHUTDOWN_TO = 0x20 // Graceful shutdown (take-over)
+      ,BUCKET_CREATED_SELF  = 0x10 // New nodegroup (me)
+      ,BUCKET_CREATED_OTHER = 0x20 // New nodegroup (not me)
+      ,BUCKET_CREATED_MASK  = (BUCKET_CREATED_SELF | BUCKET_CREATED_OTHER)
+      ,BUCKET_DROPPED_SELF  = 0x40 // New nodegroup (me) uses hi 8 bit for cnt
+      ,BUCKET_DROPPED_OTHER = 0x80 // New nodegroup (not me)
+      ,BUCKET_DROPPED_MASK  = (BUCKET_DROPPED_SELF | BUCKET_DROPPED_OTHER)
+      ,BUCKET_SHUTDOWN = 0x100 // Graceful shutdown
+      ,BUCKET_SHUTDOWN_TO = 0x200 // Graceful shutdown
     };
     Uint16 m_state;
     Uint16 m_switchover_node;
@@ -613,7 +638,6 @@ private:
   Bucket_mask m_active_buckets;
   Bucket_mask m_switchover_buckets;  
   
-  class Dbtup* m_tup;
   void init_buffers();
   Uint32* get_buffer_ptr(Signal*, Uint32 buck, Uint64 gci, Uint32 sz);
   Uint32 seize_page();
@@ -628,8 +652,7 @@ private:
 
   Uint64 get_current_gci(Signal*);
 
-  void checkMaxBufferedEpochs
-(Signal *signal);
+  void checkMaxBufferedEpochs(Signal *signal);
 
   Uint64 m_max_seen_gci;      // FIRE_TRIG_ORD
   Uint64 m_max_sent_gci;      // FIRE_TRIG_ORD -> send
@@ -653,6 +676,9 @@ private:
 
   struct Page_chunk
   {
+    STATIC_CONST( CHUNK_PAGE_SIZE = 32768 );
+    STATIC_CONST( PAGES_PER_CHUNK = 16 );
+
     Uint32 m_page_id;
     Uint32 m_size;
     Uint32 m_free;
@@ -665,13 +691,34 @@ private:
 
   Uint32 m_first_free_page;
   ArrayPool<Page_chunk> c_page_chunk_pool;
+  ArrayPool<Buffer_page> c_page_pool;
 
 #ifdef VM_TRACE
   Uint64 m_gcp_monitor;
 #endif
 
+  struct SubGcpCompleteCounter
+  {
+    Uint64 m_gci;
+    Uint32 m_cnt;
+  };
+
+  Uint32 m_gcp_rep_cnt;
+  Uint32 m_min_gcp_rep_counter_index;
+  Uint32 m_max_gcp_rep_counter_index;
+  struct SubGcpCompleteCounter m_gcp_rep_counter[10];
+
+  /* Buffer used in Suma::execALTER_TAB_REQ(). */
+  Uint32 b_dti_buf[MAX_WORDS_META_FILE];
+  Uint64 m_current_gci;
+
   Uint32 m_startphase;
   Uint32 m_typeOfStart;
+
+  void sendScanSubTableData(Signal* signal, Ptr<SyncRecord>, Uint32);
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
