@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include "Trix.hpp"
 
@@ -40,6 +40,9 @@
 #include <signaldata/TransIdAI.hpp>
 #include <signaldata/WaitGCP.hpp>
 
+#define JAM_FILE_ID 433
+
+
 #define CONSTRAINT_VIOLATION 893
 
 static
@@ -146,10 +149,10 @@ Trix::execREAD_CONFIG_REQ(Signal* signal)
 
   DLList<SubscriptionRecord> subscriptions(c_theSubscriptionRecPool);
   SubscriptionRecPtr subptr;
-  while(subscriptions.seize(subptr) == true) {
+  while (subscriptions.seizeFirst(subptr) == true) {
     new (subptr.p) SubscriptionRecord(c_theAttrOrderBufferPool);
   }
-  subscriptions.release();
+  while (subscriptions.releaseFirst());
 
   ReadConfigConf * conf = (ReadConfigConf*)signal->getDataPtrSend();
   conf->senderRef = reference();
@@ -224,7 +227,8 @@ void Trix::execREAD_NODESCONF(Signal* signal)
     if(NdbNodeBitmask::get(readNodes->allNodes, i)) {
       // Node is defined
       jam();
-      ndbrequire(c_theNodes.seizeId(nodeRecPtr, i));
+      ndbrequire(c_theNodes.getPool().seizeId(nodeRecPtr, i));
+      c_theNodes.addFirst(nodeRecPtr);
       nodeRecPtr.p->trixRef = calcTrixBlockRef(i);
       if (i == c_masterNodeId) {
         c_masterTrixRef = nodeRecPtr.p->trixRef;
@@ -579,7 +583,7 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
   SubscriptionRecord* subRec;
   SectionHandle handle(this, signal);
 
-  if (!c_theSubscriptions.seizeId(subRecPtr, buildIndxReq->buildId)) {
+  if (!c_theSubscriptions.getPool().seizeId(subRecPtr, buildIndxReq->buildId)) {
     jam();
     // Failed to allocate subscription record
     BuildIndxRef* buildIndxRef = (BuildIndxRef*)signal->getDataPtrSend();
@@ -590,6 +594,7 @@ void Trix:: execBUILD_INDX_IMPL_REQ(Signal* signal)
                BuildIndxRef::SignalLength, JBB);
     DBUG_VOID_RETURN;
   }
+  c_theSubscriptions.addFirst(subRecPtr);
 
   subRec = subRecPtr.p;
   subRec->errorCode = BuildIndxRef::NoError;
@@ -1451,7 +1456,7 @@ Trix::execCOPY_DATA_IMPL_REQ(Signal* signal)
   SubscriptionRecPtr subRecPtr;
   SectionHandle handle(this, signal);
 
-  if (!c_theSubscriptions.seize(subRecPtr))
+  if (!c_theSubscriptions.seizeFirst(subRecPtr))
   {
     jam();
     // Failed to allocate subscription record
@@ -1599,7 +1604,7 @@ Trix::statOpSeize(Uint32& statPtrI)
 
   SubscriptionRecPtr subRecPtr;
   if (ERROR_INSERTED(18002) ||
-      !c_theSubscriptions.seize(subRecPtr))
+      !c_theSubscriptions.seizeFirst(subRecPtr))
   {
     jam();
     CLEAR_ERROR_INSERT_VALUE;
