@@ -1,6 +1,5 @@
 /*
-   Copyright (c) 2003-2006 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +12,20 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #ifndef BLOCK_CALLBACK_HPP
 #define BLOCK_CALLBACK_HPP
 
+#define JAM_FILE_ID 223
+
+
 /**
  * Block callbacks
  */
 
+
+#undef JAM_FILE_ID
+
 #endif
