@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2014 Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,11 +15,11 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include <ndb_global.h>
 #include <NdbDictionary.hpp>
 #include "NdbIndexScanOperation.hpp"
 #include "NdbQueryBuilder.hpp"
 #include "NdbQueryBuilderImpl.hpp"
-#include <ndb_global.h>
 #include <Vector.hpp>
 #include "signaldata/QueryTree.hpp"
 
@@ -53,6 +53,7 @@ static const bool doPrintQueryTree = false;
 
 /* Various error codes that are not specific to NdbQuery. */
 static const int Err_MemoryAlloc = 4000;
+static const int Err_FunctionNotImplemented = 4003;
 static const int Err_UnknownColumn = 4004;
 static const int Err_FinaliseNotCalled = 4519;
 
@@ -703,6 +704,13 @@ NdbQueryBuilder* NdbQueryBuilder::create()
   {
     if (likely(impl->getNdbError().code == 0))
     {
+      if((!ndb_join_pushdown(ndbGetOwnVersion())))
+      {
+        /* The SPJ code is present in releases where the SPJ feature is
+         * not yet enabled.
+         */
+        impl->setErrorCode(Err_FunctionNotImplemented);
+      }
       return &impl->m_interface;
     }
     else
