@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2004, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,17 +12,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include <ndb_global.h>
-#include <my_pthread.h>
-#include <my_sys.h>
 
 #include "ndb_cluster_connection_impl.hpp"
 #include <mgmapi_configuration.hpp>
 #include <mgmapi_config_parameters.h>
-#include <TransporterFacade.hpp>
+#include "TransporterFacade.hpp"
 #include <NdbOut.hpp>
 #include <NdbSleep.h>
 #include <NdbThread.h>
@@ -31,7 +29,8 @@
 #include <ndb_version.h>
 #include <mgmapi_debug.h>
 #include <mgmapi_internal.h>
-#include <md5_hash.hpp>
+#include "NdbImpl.hpp"
+#include "NdbDictionaryImpl.hpp"
 
 #include <NdbMutex.h>
 #ifdef VM_TRACE
@@ -46,16 +45,23 @@ static int g_ndb_connection_count = 0;
 /*
  * Ndb_cluster_connection
  */
-
 Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string)
-  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0))
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0, 0))
+{
+}
+
+Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string,
+                                               int force_api_nodeid)
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string, 0,
+                                             force_api_nodeid))
 {
 }
 
 Ndb_cluster_connection::Ndb_cluster_connection(const char *connect_string,
                                                Ndb_cluster_connection *
                                                main_connection)
-  : m_impl(* new Ndb_cluster_connection_impl(connect_string, main_connection))
+  : m_impl(* new Ndb_cluster_connection_impl(connect_string,
+                                             main_connection, 0))
 {
 }
 
@@ -93,7 +99,9 @@ const char *Ndb_cluster_connection::get_connectstring(char *buf,
   return 0;
 }
 
-pthread_handler_t run_ndb_cluster_connection_connect_thread(void *me)
+extern "C"
+void *
+run_ndb_cluster_connection_connect_thread(void *me)
 {
   Ndb_cluster_connection_impl* connection= (Ndb_cluster_connection_impl*) me;
   connection->m_run_connect_thread= 1;
@@ -153,7 +161,7 @@ Ndb_cluster_connection_impl::get_next_node(Ndb_cluster_connection_node_iter &ite
   if (cur_pos >= no_db_nodes())
     return 0;
 
-  Ndb_cluster_connection_impl::Node *nodes= m_impl.m_all_nodes.getBase();
+  Ndb_cluster_connection_impl::Node *nodes= m_all_nodes.getBase();
   Ndb_cluster_connection_impl::Node &node=  nodes[cur_pos];
 
   if (iter.scan_state != (Uint8)~0)
@@ -219,6 +227,38 @@ Ndb_cluster_connection::node_id()
   return m_impl.m_transporter_facade->ownId();
 }
 
+unsigned
+Ndb_cluster_connection::max_nodegroup()
+{
+  TransporterFacade *tp = m_impl.m_transporter_facade;
+  if (tp == 0 || tp->ownId() == 0)
+    return 0;
+
+  Bitmask<MAX_NDB_NODES> ng;
+  tp->lock_mutex();
+  for(unsigned i= 0; i < no_db_nodes(); i++)
+  {
+    //************************************************
+    // If any node is answering, ndb is answering
+    //************************************************
+    trp_node n = tp->theClusterMgr->getNodeInfo(m_impl.m_all_nodes[i].id);
+    if (n.is_confirmed() && n.m_state.nodeGroup <= MAX_NDB_NODES)
+      ng.set(n.m_state.nodeGroup);
+  }
+  tp->unlock_mutex();
+
+  if (ng.isclear())
+    return 0;
+
+  Uint32 n = ng.find_first();
+  Uint32 m;
+  do
+  {
+    m = n;
+  } while ((n = ng.find(n+1)) != ng.NotFound);
+
+  return m;
+}
 
 int Ndb_cluster_connection::get_no_ready()
 {
@@ -291,6 +331,11 @@ unsigned Ndb_cluster_connection::get_connect_count() const
   return m_impl.get_connect_count();
 }
 
+unsigned Ndb_cluster_connection::get_min_db_version() const
+{
+  return m_impl.get_min_db_version();
+}
+
 int Ndb_cluster_connection::get_latest_error() const
 {
   return m_impl.m_latest_error;
@@ -306,19 +351,19 @@ const char *Ndb_cluster_connection::get_latest_error_msg() const
  */
 
 Ndb_cluster_connection_impl::
-Ndb_cluster_connection_impl(const char *
-                            connect_string,
-                            Ndb_cluster_connection *main_connection)
+Ndb_cluster_connection_impl(const char * connect_string,
+                            Ndb_cluster_connection *main_connection,
+                            int force_api_nodeid)
   : Ndb_cluster_connection(*this),
     m_main_connection(main_connection),
     m_optimized_node_selection(1),
-    m_name(0),
     m_run_connect_thread(0),
-    m_event_add_drop_mutex(0),
     m_latest_trans_gci(0),
     m_first_ndb_object(0),
     m_latest_error_msg(),
-    m_latest_error(0)
+    m_latest_error(0),
+    m_max_trans_id(0),
+    m_multi_wait_group(0)
 {
   DBUG_ENTER("Ndb_cluster_connection");
   DBUG_PRINT("enter",("Ndb_cluster_connection this=0x%lx", (long) this));
@@ -336,23 +381,26 @@ Ndb_cluster_connection_impl(const char *
       of order.  Same applies for regular ndbapi user.
     */
     g_eventLogger->setRepeatFrequency(0);
+
+#ifdef VM_TRACE
+    ndb_print_state_mutex= NdbMutex_Create();
+#endif
+
   }
   NdbMutex_Unlock(g_ndb_connection_mutex);
 
-  if (!m_event_add_drop_mutex)
-    m_event_add_drop_mutex= NdbMutex_Create();
+  m_event_add_drop_mutex= NdbMutex_Create();
   m_new_delete_ndb_mutex = NdbMutex_Create();
 
-
   m_connect_thread= 0;
   m_connect_callback= 0;
 
-#ifdef VM_TRACE
-  if (ndb_print_state_mutex == NULL)
-    ndb_print_state_mutex= NdbMutex_Create();
-#endif
+  /* Clear global stats baseline */
+  memset(globalApiStatsBaseline, 0, sizeof(globalApiStatsBaseline));
+
   m_config_retriever=
-    new ConfigRetriever(connect_string, NDB_VERSION, NODE_TYPE_API);
+    new ConfigRetriever(connect_string, force_api_nodeid,
+                        NDB_VERSION, NDB_MGM_NODE_TYPE_API);
   if (m_config_retriever->hasError())
   {
     m_latest_error= 1;
@@ -360,13 +408,6 @@ Ndb_cluster_connection_impl(const char *
       ("Could not initialize handle to management server: %s",
        m_config_retriever->getErrorString());
     printf("%s\n", get_latest_error_msg());
-    delete m_config_retriever;
-    m_config_retriever= 0;
-  }
-  if (m_name)
-  {
-    NdbMgmHandle h= m_config_retriever->get_mgmHandle();
-    ndb_mgm_set_name(h, m_name);
   }
   if (!m_main_connection)
   {
@@ -379,11 +420,13 @@ Ndb_cluster_connection_impl(const char *
     m_globalDictCache = 0;
     m_transporter_facade=
       new TransporterFacade(m_main_connection->m_impl.m_globalDictCache);
+
+    // The secondary connection can't use same nodeid, clear the nodeid
+    // in ConfigRetriver to avoid asking for the same nodeid again
+    m_config_retriever->setNodeId(0);
+
   }
 
-
-
-
   DBUG_VOID_RETURN;
 }
 
@@ -432,29 +475,32 @@ Ndb_cluster_connection_impl::~Ndb_cluster_connection_impl()
     delete m_config_retriever;
     m_config_retriever= NULL;
   }
-#ifdef VM_TRACE
-  if (ndb_print_state_mutex != NULL)
-  {
-    NdbMutex_Destroy(ndb_print_state_mutex);
-    ndb_print_state_mutex= NULL;
-  }
-#endif
-  if (m_name)
-    free(m_name);
 
   NdbMutex_Lock(g_ndb_connection_mutex);
   if(--g_ndb_connection_count == 0)
   {
     NdbColumnImpl::destory_pseudo_columns();
+
+#ifdef VM_TRACE
+    NdbMutex_Destroy(ndb_print_state_mutex);
+    ndb_print_state_mutex= NULL;
+#endif
+
   }
   NdbMutex_Unlock(g_ndb_connection_mutex);
 
   if (m_event_add_drop_mutex)
     NdbMutex_Destroy(m_event_add_drop_mutex);
+  m_event_add_drop_mutex = 0;
 
-  NdbMutex_Destroy(m_new_delete_ndb_mutex);
+  if (m_new_delete_ndb_mutex)
+    NdbMutex_Destroy(m_new_delete_ndb_mutex);
   m_new_delete_ndb_mutex = 0;
   
+  if(m_multi_wait_group)
+    delete m_multi_wait_group;
+  m_multi_wait_group = 0;
+
   DBUG_VOID_RETURN;
 }
 
@@ -491,6 +537,7 @@ Ndb_cluster_connection_impl::link_ndb_object(Ndb* p)
   p->theImpl->m_next_ndb_object = m_first_ndb_object;
   m_first_ndb_object = p;
   
+  p->theFirstTransId += m_max_trans_id;
   unlock_ndb_objects();
 }
 
@@ -519,20 +566,29 @@ Ndb_cluster_connection_impl::unlink_ndb_object(Ndb* p)
   p->theImpl->m_prev_ndb_object = 0;
   p->theImpl->m_next_ndb_object = 0;
 
+  Uint32 transId = (Uint32)p->theFirstTransId;
+  if (transId > m_max_trans_id)
+  {
+    m_max_trans_id = transId;
+  }
+
+  /* This Ndb is leaving for a better place,
+   * record its contribution to global warming
+   * for posterity
+   */
+  for (Uint32 i=0; i<Ndb::NumClientStatistics; i++)
+  {
+    globalApiStatsBaseline[i] += p->theImpl->clientStats[i];
+  }
+
   unlock_ndb_objects();  
 }
 
 void
 Ndb_cluster_connection_impl::set_name(const char *name)
 {
-  if (m_name)
-    free(m_name);
-  m_name= strdup(name);
-  if (m_config_retriever && m_name)
-  {
-    NdbMgmHandle h= m_config_retriever->get_mgmHandle();
-    ndb_mgm_set_name(h, m_name);
-  }
+  NdbMgmHandle h= m_config_retriever->get_mgmHandle();
+  ndb_mgm_set_name(h, name);
 }
 
 int
@@ -581,52 +637,52 @@ Ndb_cluster_connection_impl::init_nodes_vector(Uint32 nodeid,
       break;
     }
     }
-    if (m_impl.m_all_nodes.push_back(Node(group,remoteNodeId)))
+    if (m_all_nodes.push_back(Node(group,remoteNodeId)))
     {
       DBUG_RETURN(-1);
     }
     DBUG_PRINT("info",("saved %d %d", group,remoteNodeId));
-    for (int i= m_impl.m_all_nodes.size()-2;
-	 i >= 0 && m_impl.m_all_nodes[i].group > m_impl.m_all_nodes[i+1].group;
+    for (int i= m_all_nodes.size()-2;
+	 i >= 0 && m_all_nodes[i].group > m_all_nodes[i+1].group;
 	 i--)
     {
-      Node tmp= m_impl.m_all_nodes[i];
-      m_impl.m_all_nodes[i]= m_impl.m_all_nodes[i+1];
-      m_impl.m_all_nodes[i+1]= tmp;
+      Node tmp= m_all_nodes[i];
+      m_all_nodes[i]= m_all_nodes[i+1];
+      m_all_nodes[i+1]= tmp;
     }
   }
 
   int i;
   Uint32 cur_group, i_group= 0;
   cur_group= ~0;
-  for (i= (int)m_impl.m_all_nodes.size()-1; i >= 0; i--)
+  for (i= (int)m_all_nodes.size()-1; i >= 0; i--)
   {
-    if (m_impl.m_all_nodes[i].group != cur_group)
+    if (m_all_nodes[i].group != cur_group)
     {
-      cur_group= m_impl.m_all_nodes[i].group;
+      cur_group= m_all_nodes[i].group;
       i_group= i+1;
     }
-    m_impl.m_all_nodes[i].next_group= i_group;
+    m_all_nodes[i].next_group= i_group;
   }
   cur_group= ~0;
-  for (i= 0; i < (int)m_impl.m_all_nodes.size(); i++)
+  for (i= 0; i < (int)m_all_nodes.size(); i++)
   {
-    if (m_impl.m_all_nodes[i].group != cur_group)
+    if (m_all_nodes[i].group != cur_group)
     {
-      cur_group= m_impl.m_all_nodes[i].group;
+      cur_group= m_all_nodes[i].group;
       i_group= i;
     }
-    m_impl.m_all_nodes[i].this_group= i_group;
+    m_all_nodes[i].this_group= i_group;
   }
 #if 0
-  for (i= 0; i < (int)m_impl.m_all_nodes.size(); i++)
+  for (i= 0; i < (int)m_all_nodes.size(); i++)
   {
     fprintf(stderr, "[%d] %d %d %d %d\n",
 	   i,
-	   m_impl.m_all_nodes[i].id,
-	   m_impl.m_all_nodes[i].group,
-	   m_impl.m_all_nodes[i].this_group,
-	   m_impl.m_all_nodes[i].next_group);
+	   m_all_nodes[i].id,
+	   m_all_nodes[i].group,
+	   m_all_nodes[i].this_group,
+	   m_all_nodes[i].next_group);
   }
 
   do_test();
@@ -634,6 +690,97 @@ Ndb_cluster_connection_impl::init_nodes_vector(Uint32 nodeid,
   DBUG_RETURN(0);
 }
 
+Uint32
+Ndb_cluster_connection_impl::get_db_nodes(Uint8 arr[MAX_NDB_NODES]) const
+{
+  Uint32 cnt = (Uint32)m_all_nodes.size();
+  assert(cnt < MAX_NDB_NODES);
+  const Node *nodes = m_all_nodes.getBase();
+  for (Uint32 i = 0; i<cnt; i++)
+    arr[i] = (Uint8)nodes[i].id;
+  return cnt;
+}
+
+int
+Ndb_cluster_connection_impl::configure(Uint32 nodeId,
+                                       const ndb_mgm_configuration &config)
+{
+  DBUG_ENTER("Ndb_cluster_connection_impl::configure");
+  {
+    ndb_mgm_configuration_iterator iter(config, CFG_SECTION_NODE);
+    if(iter.find(CFG_NODE_ID, nodeId))
+      DBUG_RETURN(-1);
+
+    // Configure scan settings
+    Uint32 scan_batch_size= 0;
+    if (!iter.get(CFG_MAX_SCAN_BATCH_SIZE, &scan_batch_size)) {
+      m_config.m_scan_batch_size= scan_batch_size;
+    }
+    Uint32 batch_byte_size= 0;
+    if (!iter.get(CFG_BATCH_BYTE_SIZE, &batch_byte_size)) {
+      m_config.m_batch_byte_size= batch_byte_size;
+    }
+    Uint32 batch_size= 0;
+    if (!iter.get(CFG_BATCH_SIZE, &batch_size)) {
+      m_config.m_batch_size= batch_size;
+    }
+
+    Uint32 queue = 0;
+    if (!iter.get(CFG_DEFAULT_OPERATION_REDO_PROBLEM_ACTION, &queue))
+    {
+      m_config.m_default_queue_option = queue;
+    }
+
+    Uint32 default_hashmap_size = 0;
+    if (!iter.get(CFG_DEFAULT_HASHMAP_SIZE, &default_hashmap_size) &&
+        default_hashmap_size != 0)
+    {
+      m_config.m_default_hashmap_size = default_hashmap_size;
+    }
+    // If DefaultHashmapSize is not set or zero, use the minimum
+    // value set (not zero) for any other node, since this size
+    // should be supported by the other nodes.  Also this allows
+    // the DefaultHashmapSize to be set for the entire cluster
+    // if set for a single node or node type.
+    // Otherwise use NDB_DEFAULT_HASHMAP_BUCKETS
+    if (default_hashmap_size == 0)
+    {
+      // Use new iterator to leave iter valid.
+      ndb_mgm_configuration_iterator iterall(config, CFG_SECTION_NODE);
+      for (; iterall.valid(); iterall.next())
+      {
+        Uint32 tmp = 0;
+        if (!iterall.get(CFG_DEFAULT_HASHMAP_SIZE, &tmp) &&
+            tmp != 0 &&
+            ((default_hashmap_size == 0) || (tmp < default_hashmap_size)))
+          default_hashmap_size = tmp;
+      }
+      if (default_hashmap_size == 0)
+        default_hashmap_size = NDB_DEFAULT_HASHMAP_BUCKETS;
+
+      m_config.m_default_hashmap_size = default_hashmap_size;
+    }
+
+    // Configure timeouts
+    {
+      Uint32 timeout = 120000;
+      // Use new iterator to leave iter valid.
+      ndb_mgm_configuration_iterator iterall(config, CFG_SECTION_NODE);
+      for (; iterall.valid(); iterall.next())
+      {
+        Uint32 tmp1 = 0, tmp2 = 0;
+        iterall.get(CFG_DB_TRANSACTION_CHECK_INTERVAL, &tmp1);
+        iterall.get(CFG_DB_TRANSACTION_DEADLOCK_TIMEOUT, &tmp2);
+        tmp1 += tmp2;
+        if (tmp1 > timeout)
+          timeout = tmp1;
+      }
+      m_config.m_waitfor_timeout = timeout;
+    }
+  }
+  DBUG_RETURN(init_nodes_vector(nodeId, config));
+}
+
 void
 Ndb_cluster_connection_impl::do_test()
 {
@@ -691,85 +838,101 @@ void Ndb_cluster_connection::set_name(const char *name)
   m_impl.set_name(name);
 }
 
-int Ndb_cluster_connection::connect(int no_retries, int retry_delay_in_seconds,
-				    int verbose)
+int Ndb_cluster_connection_impl::connect(int no_retries,
+                                         int retry_delay_in_seconds,
+                                         int verbose)
 {
-  struct ndb_mgm_reply mgm_reply;
-
   DBUG_ENTER("Ndb_cluster_connection::connect");
   do {
-    if (m_impl.m_config_retriever == 0)
+    if (m_config_retriever == 0)
     {
-      if (!m_impl.m_latest_error)
+      if (!m_latest_error)
       {
-        m_impl.m_latest_error= 1;
-        m_impl.m_latest_error_msg.assign
-          ("Ndb_cluster_connection init error: m_impl.m_config_retriever==0");
+        m_latest_error = 1;
+        m_latest_error_msg.assign("Ndb_cluster_connection init "
+                                  "error: m_config_retriever==0");
       }
-      DBUG_PRINT("exit", ("no m_impl.m_config_retriever, ret: -1"));
+      DBUG_PRINT("exit", ("no m_config_retriever, ret: -1"));
       DBUG_RETURN(-1);
     }
-    if (m_impl.m_config_retriever->do_connect(no_retries,
-					      retry_delay_in_seconds,
-					      verbose))
+
+    // the allocNodeId function will connect if not connected
+    int alloc_error;
+    Uint32 nodeId = m_config_retriever->allocNodeId(no_retries,
+                                                   retry_delay_in_seconds,
+                                                   verbose, alloc_error);
+    if (!nodeId)
     {
-      char buf[1024];
-      m_impl.m_latest_error= 1;
-      m_impl.m_latest_error_msg.assfmt
-        ("Connect using '%s' timed out", get_connectstring(buf, sizeof(buf)));
-      DBUG_PRINT("exit", ("mgmt server not up yet, ret: 1"));
-      DBUG_RETURN(1); // mgmt server not up yet
+      // Failed to allocate nodeid from mgmt server, find out
+      // the cause and set proper error message
+
+      if (!m_config_retriever->is_connected())
+      {
+        // Could not connect to mgmt server
+        m_latest_error = alloc_error;
+        m_latest_error_msg.assfmt("%s", m_config_retriever->getErrorString());
+        DBUG_RETURN(1); // Recoverable error
+      }
+
+      if (alloc_error == NDB_MGM_ALLOCID_ERROR)
+      {
+        // A nodeid for this node was found in config, but it was not
+        // free right now. Retry later and it might be free.
+        m_latest_error = alloc_error;
+        m_latest_error_msg.assfmt("%s", m_config_retriever->getErrorString());
+        DBUG_RETURN(1); // Recoverable error
+      }
+
+      // Fatal error, use default error
+      break;
     }
 
-    Uint32 nodeId = m_impl.m_config_retriever->allocNodeId(4/*retries*/,
-							   3/*delay*/);
-    if(nodeId == 0)
-      break;
-    ndb_mgm_configuration * props = m_impl.m_config_retriever->getConfig();
+    ndb_mgm_configuration * props = m_config_retriever->getConfig(nodeId);
     if(props == 0)
       break;
 
-    m_impl.m_transporter_facade->start_instance(nodeId, props);
-    if (m_impl.init_nodes_vector(nodeId, *props))
+    if (configure(nodeId, *props))
     {
-      ndbout_c("Ndb_cluster_connection::connect: malloc failure");
+      ndb_mgm_destroy_configuration(props);
       DBUG_PRINT("exit", ("malloc failure, ret: -1"));
       DBUG_RETURN(-1);
     }
 
-    for(unsigned i=0;
-	i<m_impl.m_transporter_facade->get_registry()->m_transporter_interface.size();
-	i++)
-      ndb_mgm_set_connection_int_parameter(m_impl.m_config_retriever->get_mgmHandle(),
-					   nodeId,
-					   m_impl.m_transporter_facade->get_registry()
-					     ->m_transporter_interface[i]
-					     .m_remote_nodeId,
-					   CFG_CONNECTION_SERVER_PORT,
-					   m_impl.m_transporter_facade->get_registry()
-					     ->m_transporter_interface[i]
-					     .m_s_service_port,
-					   &mgm_reply);
+    if (m_transporter_facade->start_instance(nodeId, props) < 0)
+    {
+      ndb_mgm_destroy_configuration(props);
+      DBUG_RETURN(-1);
+    }
 
     ndb_mgm_destroy_configuration(props);
-    m_impl.m_transporter_facade->connected();
-    m_impl.m_latest_error= 0;
-    m_impl.m_latest_error_msg.assign("");
+    m_transporter_facade->connected();
+    m_latest_error = 0;
+    m_latest_error_msg.assign("");
     DBUG_PRINT("exit", ("connect ok, ret: 0"));
     DBUG_RETURN(0);
   } while(0);
-  
-  const char* erString = m_impl.m_config_retriever->getErrorString();
+
+  const char* erString = m_config_retriever->getErrorString();
   if (erString == 0) {
     erString = "No error specified!";
   }
-  m_impl.m_latest_error= 1;
-  m_impl.m_latest_error_msg.assfmt("Configuration error: %s", erString);
+  m_latest_error = 1;
+  m_latest_error_msg.assfmt("Configuration error: %s", erString);
   ndbout << get_latest_error_msg() << endl;
   DBUG_PRINT("exit", ("connect failed, '%s' ret: -1", erString));
   DBUG_RETURN(-1);
 }
 
+
+int
+Ndb_cluster_connection::connect(int no_retries,
+                                int retry_delay_in_seconds,
+                                int verbose)
+{
+  return m_impl.connect(no_retries, retry_delay_in_seconds, verbose);
+}
+
+
 void Ndb_cluster_connection_impl::connect_thread()
 {
   DBUG_ENTER("Ndb_cluster_connection_impl::connect_thread");
@@ -840,5 +1003,149 @@ Ndb_cluster_connection::set_auto_reconnect(int value)
   m_impl.m_transporter_facade->set_auto_reconnect(value);
 }
 
+Uint32 
+Ndb_cluster_connection::collect_client_stats(Uint64* statsArr, Uint32 sz)
+{
+  /* We have a global stats baseline which contains all
+   * the stats for Ndb objects which have been and gone.
+   * Start with that, then add in stats for Ndb objects
+   * currently in use.
+   * Note that despite the lock, this is not thread safe
+   * as we are reading data that other threads may be
+   * concurrently writing.  The lock just guards against
+   * concurrent changes to the set of active Ndbs while
+   * we are iterating it.
+   */
+  const Uint32 relevant = MIN((Uint32)Ndb::NumClientStatistics, sz);
+  const Ndb* ndb = NULL;
+  lock_ndb_objects();
+  {
+    memcpy(statsArr, &m_impl.globalApiStatsBaseline[0], sizeof(Uint64)*relevant);
+  
+    while((ndb = get_next_ndb_object(ndb)) != NULL)
+    {
+      for (Uint32 i=0; i<relevant; i++)
+      {
+        statsArr[i] += ndb->theImpl->clientStats[i];
+      }
+    }
+  }
+  unlock_ndb_objects();
+
+  return relevant;
+}
+
+void
+Ndb_cluster_connection::set_max_adaptive_send_time(Uint32 milliseconds)
+{
+  m_impl.m_transporter_facade->setSendThreadInterval(milliseconds);
+}
+
+Uint32
+Ndb_cluster_connection::get_max_adaptive_send_time()
+{
+  return m_impl.m_transporter_facade->getSendThreadInterval();
+}
+
+NdbWaitGroup *
+Ndb_cluster_connection::create_ndb_wait_group(int size)
+{
+  if(m_impl.m_multi_wait_group == NULL)
+  {
+    m_impl.m_multi_wait_group = new NdbWaitGroup(this, size);
+    return m_impl.m_multi_wait_group;
+  }
+  else
+  {
+    return NULL;  // NdbWaitGroup already exists
+  }
+}
+
+bool
+Ndb_cluster_connection::release_ndb_wait_group(NdbWaitGroup *group)
+{
+  if(m_impl.m_multi_wait_group && m_impl.m_multi_wait_group == group)
+  {
+    delete m_impl.m_multi_wait_group;
+    m_impl.m_multi_wait_group = 0;
+    return true;
+  }
+  else
+  {
+    return false;
+  }
+}
+
 template class Vector<Ndb_cluster_connection_impl::Node>;
 
+int
+Ndb_cluster_connection::wait_until_ready(const int * nodes, int cnt,
+                                         int timeout)
+{
+  DBUG_ENTER("Ndb_cluster_connection::wait_until_ready(nodelist)");
+
+  NodeBitmask mask;
+  for (int i = 0; i < cnt; i++)
+  {
+    if (nodes[i] <= 0 || nodes[i] > (int)mask.max_size())
+    {
+      DBUG_RETURN(-1);
+    }
+    mask.set(nodes[i]);
+  }
+
+  TransporterFacade *tp = m_impl.m_transporter_facade;
+  if (tp == 0)
+  {
+    DBUG_RETURN(-1);
+  }
+  if (tp->ownId() == 0)
+  {
+    DBUG_RETURN(-1);
+  }
+
+  timeout *= 10; // try each 100ms
+
+  NodeBitmask dead;
+  NodeBitmask alive;
+  do
+  {
+    dead.clear();
+    alive.clear();
+    tp->lock_mutex();
+    for(unsigned i= 0; i < no_db_nodes(); i++)
+    {
+      //************************************************
+      // If any node is answering, ndb is answering
+      //************************************************
+      if (tp->get_node_alive(m_impl.m_all_nodes[i].id) != 0)
+        alive.set(m_impl.m_all_nodes[i].id);
+      else
+        dead.set(m_impl.m_all_nodes[i].id);
+    }
+    tp->unlock_mutex();
+
+    if (alive.contains(mask))
+    {
+      DBUG_RETURN(mask.count());
+    }
+
+    NodeBitmask all;
+    all.bitOR(alive);
+    all.bitOR(dead);
+    if (!all.contains(mask))
+    {
+      DBUG_RETURN(-1);
+    }
+
+    if (timeout == 0)
+      break;
+
+    timeout--;
+    NdbSleep_MilliSleep(100);
+  } while (true);
+
+  mask.bitAND(alive);
+  DBUG_RETURN(mask.count());
+}
+
