@@ -1,6 +1,5 @@
-/*
-   Copyright (c) 2007, 2008 MySQL AB, 2008 Sun Microsystems, Inc.
-   Use is subject to license terms.
+/* Copyright (c) 2007, 2008 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 // dbutil.h: interface for the database utilities class.
 // Supplies a database to the test application
@@ -71,8 +70,8 @@ public:
   uint columnAsInt(const char* col_name);
   unsigned long long columnAsLong(const char* col_name);
 
-  uint insertId();
-  uint affectedRows();
+  unsigned long long insertId();
+  unsigned long long affectedRows();
   uint numRows(void);
   uint mysqlErrno();
   const char* mysqlError();
@@ -139,7 +138,7 @@ public:
   const char* last_error() const { return m_last_error.c_str(); }
   int last_errno() const { return m_last_errno; }
 
-  unsigned long selectCountTable(const char * table);
+  unsigned long long selectCountTable(const char * table);
 
   void silent() { m_silent= true; };
 
