@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2003, 2011, 2012, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2005, 2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 
 
@@ -195,7 +195,9 @@ waitForThreads(Uint32 num_threads_to_wait_for)
     NdbSleep_MilliSleep(20);
     for (unsigned i = 0; i < num_threads_to_wait_for ; i++) {
       if (ThreadReady[i] == 0) {
+        // Found one thread not yet ready, continue waiting
         cont = 1;
+        break;
       }//if
     }//for
   } while (cont == 1);
