@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2005, 2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 
 #include <ndb_global.h>
@@ -124,7 +124,7 @@ static bool                             tReadUpdate = true;
 static int                              tUpdateFreq = 20;
 static bool                             tLocal = false;
 static int                              tLocalPart = 0;
-static int                              tMinEvents = 0;
+static Uint32                           tMinEvents = 0;
 static int                              tSendForce = 0;
 static int                              tNoOfLoops = 1;
 static Uint32                           tNoOfThreads = 1;
@@ -164,7 +164,9 @@ waitForThreads(void)
     NdbSleep_MilliSleep(20);
     for (int i = 0; i < (int)tNoOfThreads ; i++) {
       if (ThreadReady[i] == 0) {
+        // Found one thread not yet ready, continue waiting
         cont = 1;
+        break;
       }//if
     }//for
   } while (cont == 1);
@@ -184,7 +186,6 @@ NDB_COMMAND(flexTT, "flexTT", "flexTT", "flexTT", 65535)
   ndb_init();
   ThreadNdb*            pThreadData;
   int                   returnValue = NDBT_OK;
-  int i;
   flexTTErrorData = new ErrorData;
   flexTTErrorData->resetErrorCounters();
 
@@ -268,7 +269,7 @@ NDB_COMMAND(flexTT, "flexTT", "flexTT", "flexTT", 65535)
      *  Create NDB objects.                                   *
      ****************************************************************/
     resetThreads();
-    for (i = 0; i < (int)tNoOfThreads ; i++) {
+    for (int i = 0; i < (int)tNoOfThreads ; i++) {
       pThreadData[i].threadNo = i;
       threadLife[i] = NdbThread_Create(threadLoop,
                                        (void**)&pThreadData[i],
@@ -319,7 +320,7 @@ NDB_COMMAND(flexTT, "flexTT", "flexTT", "flexTT", 65535)
         
     execute(stStop);
     void * tmp;
-    for(i = 0; i<(int)tNoOfThreads; i++){
+    for(int i = 0; i<(int)tNoOfThreads; i++){
       NdbThread_WaitFor(threadLife[i], &tmp);
       NdbThread_Destroy(&threadLife[i]);
     }
@@ -835,7 +836,7 @@ readArguments(int argc, const char** argv){
       }
     } else if (strcmp(argv[i], "-e") == 0){
       tMinEvents = atoi(argv[i+1]);
-      if ((tMinEvents < 1) || (tMinEvents > (int)tNoOfParallelTrans)) {
+      if ((tMinEvents < 1) || (tMinEvents > tNoOfParallelTrans)) {
 	ndbout_c("Invalid no of loops");
         return -1;
       }
