@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2006 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include <NDBT.hpp>
 #include <NDBT_Test.hpp>
 #include <HugoTransactions.hpp>
 #include <UtilTransactions.hpp>
 #include <NdbBackup.hpp>
+#include <NdbMgmd.hpp>
+#include <signaldata/DumpStateOrd.hpp>
 
 int runDropTable(NDBT_Context* ctx, NDBT_Step* step);
 
@@ -673,6 +675,249 @@ runBug57650(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
+int
+runBug14019036(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbBackup backup;
+  NdbRestarter res;
+  NdbMgmd mgmd;
+
+  res.insertErrorInAllNodes(5073); // slow down backup
+
+  if (!mgmd.connect()) {
+    g_err << "Cannot connect to mgmd server" << endl;
+    return NDBT_FAILED;
+  }
+  if (!mgmd.subscribe_to_events()) {
+    g_err << "Cannot subscribe to mgmd server logevents" << endl;
+    return NDBT_FAILED;
+  }
+  Uint64 maxWaitSeconds = 10;
+  Uint64 endTime = NdbTick_CurrentMillisecond() +
+                   (maxWaitSeconds * 1000);
+
+  int val2[] = { 100000 }; // all dump 100000 
+  unsigned backupId = 0;
+  if (backup.start(backupId, 1, 0, 1) == -1) {
+    g_err << "Failed to start backup nowait" << endl;
+    return NDBT_FAILED;
+  }
+  int records = 0, data = 0;
+  int result = NDBT_OK;
+  while (NdbTick_CurrentMillisecond() < endTime)
+  {
+    char buff[512];
+    char tmp[512];
+
+    // dump backup status in mgmd log 
+    res.dumpStateAllNodes(val2, 1);
+
+    // read backup status logevent from mgmd
+    if (!mgmd.get_next_event_line(buff, sizeof(buff), 10 * 1000)) {
+      g_err << "Failed to read logevent from mgmd" << endl;
+      return NDBT_FAILED;
+    }
+    if(strstr(buff, "#Records")) 
+       sscanf(buff, "%s %d", tmp, &records);
+    if(strstr(buff, "Data")) {
+      sscanf(buff, "%s %d", tmp, &data);
+      if(records == 0 && data > 0) {
+        g_err << "Inconsistent backup status: ";
+        g_err << "Data written = " << data << " bytes, Record count = 0" << endl;
+        result = NDBT_FAILED;
+        break;
+      }
+      else if(records > 0 && data > 0)
+        break;
+    }
+  }    
+
+  res.insertErrorInAllNodes(0);
+
+  return result;
+}
+int
+runBug16656639(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbBackup backup;
+  NdbRestarter res;
+
+  res.insertErrorInAllNodes(10032); 
+
+  g_err << "Dumping schema state." << endl;
+
+  int dump1 = DumpStateOrd::SchemaResourceSnapshot;
+  int dump2 = DumpStateOrd::SchemaResourceCheckLeak;
+  res.dumpStateAllNodes(&dump1, 1);
+
+  g_err << "Starting backup." << endl;
+  unsigned backupId = 0;
+  if (backup.start(backupId, 1, 0, 1) == -1) {
+    g_err << "Failed to start backup." << endl;
+    return NDBT_FAILED;
+  }
+
+  g_err << "Waiting 1 sec for frag scans to start." << endl;
+  NdbSleep_SecSleep(1);
+
+  g_err << "Aborting backup." << endl;
+  if(backup.abort(backupId) == -1) {
+    g_err << "Failed to abort backup." << endl;
+    return NDBT_FAILED;
+  }
+
+  g_err << "Checking backup status." << endl;
+  if(backup.startLogEvent() != 0) {
+    g_err << "Can't create log event." << endl;
+    return NDBT_FAILED;
+  }
+  if(backup.checkBackupStatus() != 3) {
+    g_err << "Backup not aborted." << endl;
+    return NDBT_FAILED;
+  }
+
+  res.insertErrorInAllNodes(0);
+  if(res.dumpStateAllNodes(&dump2, 1) != 0) {
+    g_err << "Schema leak." << endl;
+    return NDBT_FAILED;
+  }
+  return NDBT_OK;
+}
+
+
+int makeTmpTable(NdbDictionary::Table& tab, NdbDictionary::Index &idx, const char *tableName, const char *columnName)
+{
+  tab.setName(tableName);
+  tab.setLogging(true);
+  {
+    // create column
+    NdbDictionary::Column col(columnName);
+    col.setType(NdbDictionary::Column::Unsigned);
+    col.setPrimaryKey(true);
+    tab.addColumn(col);
+
+    // create index on column
+    idx.setTable(tableName);
+    idx.setName("idx1");
+    idx.setType(NdbDictionary::Index::OrderedIndex);
+    idx.setLogging(false);
+    idx.addColumnName(columnName);
+  }
+  NdbError error;
+  return tab.validate(error);
+} 
+
+int
+runBug17882305(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbBackup backup;
+  NdbDictionary::Table tab;
+  NdbDictionary::Index idx;
+  const char *tablename = "#sql-dummy"; 
+  const char *colname = "_id"; 
+  
+  Ndb* const ndb = GETNDB(step);
+  NdbDictionary::Dictionary* const dict = ndb->getDictionary();
+
+  // create "#sql-dummy" table
+  if(makeTmpTable(tab, idx, tablename, colname) == -1) {
+    g_err << "Validation of #sql table failed" << endl;
+    return NDBT_FAILED;
+  }
+  if (dict->createTable(tab) == -1) {
+    g_err << "Failed to create #sql table." << endl;
+    return NDBT_FAILED;
+  }
+  if (dict->createIndex(idx) == -1) {
+    g_err << "Failed to create index, error: " << dict->getNdbError() << endl;
+    return NDBT_FAILED;
+  }
+
+  // start backup which will contain "#sql-dummy"
+  g_err << "Starting backup." << endl;
+  unsigned backupId = 0;
+  if (backup.start(backupId, 2, 0, 1) == -1) {
+    g_err << "Failed to start backup." << endl;
+    return NDBT_FAILED;
+  }
+
+  // drop "#sql-dummy"
+  if (dict->dropTable(tablename) == -1) {
+    g_err << "Failed to drop #sql-dummy table." << endl;
+    return NDBT_FAILED;
+  }
+
+  // restore from backup, data only.
+  // backup contains data for #sql-dummy, which should 
+  // cause an error as the table doesn't exist, but will 
+  // not cause an error as the default value for 
+  // --exclude-intermediate-sql-tables is 1
+  if (backup.restore(backupId, false) != 0) {
+    g_err << "Failed to restore from backup." << endl;
+    return NDBT_FAILED;
+  }
+
+  return NDBT_OK;
+}   
+
+int
+runBug19202654(NDBT_Context* ctx, NDBT_Step* step)
+{
+  NdbBackup backup;
+  NdbDictionary::Dictionary* const dict = GETNDB(step)->getDictionary();
+
+  g_err << "Creating 35 ndb tables." << endl;
+  for(int i=0; i<35; i++)
+  {
+    char tablename[10];
+    sprintf(tablename, "t%d", i);
+    const char *colname = "id";
+    NdbDictionary::Table tab;
+    NdbDictionary::Index idx;
+
+    if(makeTmpTable(tab, idx, tablename, colname) == -1) {
+      g_err << "Failed to validate table " << tablename << ", error: " << dict->getNdbError() << endl;
+      return NDBT_FAILED;
+    }
+    // Create large number of dictionary objects
+    if (dict->createTable(tab) == -1) {
+      g_err << "Failed to create table " << tablename << ", error: " << dict->getNdbError() << endl;
+      return NDBT_FAILED;
+    }
+    // Create an index per table to double the number of dictionary objects
+    if (dict->createIndex(idx) == -1) {
+      g_err << "Failed to create index, error: " << dict->getNdbError() << endl;
+      return NDBT_FAILED;
+    }
+  }
+  
+  g_err << "Starting backup." << endl;
+  unsigned backupId = 0;
+  if (backup.start(backupId) == -1) {
+    g_err << "Failed to start backup." << endl;
+    return NDBT_FAILED;
+  }
+
+  g_err << "Dropping 35 ndb tables." << endl;
+  for(int i=0; i<35; i++)
+  {
+    char tablename[10];
+    sprintf(tablename, "t%d", i);
+    if (dict->dropTable(tablename) == -1) {
+      g_err << "Failed to drop table " << tablename << ", error: " << dict->getNdbError() << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  g_err << "Restoring from backup with error insert and no metadata or data restore." << endl;
+  // just load metadata and exit
+  if (backup.restore(backupId, false, false, 1) != 0) {
+    g_err << "Failed to restore from backup." << endl;
+    return NDBT_FAILED;
+  }
+  return NDBT_OK;
+}
+
 NDBT_TESTSUITE(testBackup);
 TESTCASE("BackupOne", 
 	 "Test that backup and restore works on one table \n"
@@ -806,6 +1051,23 @@ TESTCASE("Bug57650", "")
 {
   INITIALIZER(runBug57650);
 }
+TESTCASE("Bug14019036", "")
+{
+  INITIALIZER(runBug14019036);
+}
+TESTCASE("Bug16656639", "")
+{
+  INITIALIZER(runBug16656639);
+}
+TESTCASE("Bug17882305", "")
+{
+  INITIALIZER(runBug17882305);
+}
+TESTCASE("Bug19202654", 
+         "Test restore with a large number of tables")
+{
+  INITIALIZER(runBug19202654);
+}
 NDBT_TESTSUITE_END(testBackup);
 
 int main(int argc, const char** argv){
