@@ -1,6 +1,5 @@
-/*
-   Copyright (C) 2003 MySQL AB
-    All rights reserved. Use is subject to license terms.
+/* Copyright (c) 2003-2006 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 /*
  * testDataBuffers
@@ -132,7 +131,7 @@ static bool isAligned(char* p)
 {
     return isAligned(UintPtr(p));
 }
-static unsigned toAligned(UintPtr x)
+static unsigned toAligned(unsigned x)
 {
     while (! isAligned(x))
 	x++;
@@ -373,10 +372,10 @@ testcase(Ndb_cluster_connection&cc, int flag)
 		}
 		memset(c.buf, 'B', c.bufsiz);
 		if (useBuf) {
-		    if (op->getValue(c.aAttrName, c.buf + off) < 0)
+                    if (op->getValue(c.aAttrName, c.buf + off) == NULL)
 			return ndberror("getValue key=%d col=%d", key, i);
 		} else {
-		    if ((c.aRa = op->getValue(c.aAttrName)) == 0)
+                    if ((c.aRa = op->getValue(c.aAttrName)) == NULL)
 			return ndberror("getValue key=%d col=%d", key, i);
 		}
 	    }
@@ -470,7 +469,7 @@ testcase(Ndb_cluster_connection&cc, int flag)
 	for (i = 0; i < attrcnt; i++) {
 	    col& c = ccol[i];
 	    if (i == 0) {
-		if (op->getValue(c.aAttrName, (char*)&newkey) < 0)
+                if (op->getValue(c.aAttrName, (char*)&newkey) == NULL)
 		    return ndberror("getValue key=%d col=%d", key, i);
 	    } else {
 		if (xverbose) {
@@ -483,10 +482,10 @@ testcase(Ndb_cluster_connection&cc, int flag)
 		}
 		memset(c.buf, 'C', c.bufsiz);
 		if (useBuf) {
-		    if (op->getValue(c.aAttrName, c.buf + off) < 0)
+                    if (op->getValue(c.aAttrName, c.buf + off) == NULL)
 			return ndberror("getValue key=%d col=%d", key, i);
 		} else {
-		    if ((c.aRa = op->getValue(c.aAttrName)) == 0)
+                    if ((c.aRa = op->getValue(c.aAttrName)) == NULL)
 			return ndberror("getValue key=%d col=%d", key, i);
 		}
 	    }
