@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2004, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2005 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+
 
 #include <NDBT_Test.hpp>
 #include <NDBT_ReturnCodes.h>
@@ -21,11 +21,9 @@
 #include <UtilTransactions.hpp>
 #include <NdbRestarter.hpp>
 
-#define GETNDB(ps) ((NDBT_NdbApiStep*)ps)->getNdb()
-
 static Uint32 max_dks = 0;
 static const Uint32 MAX_FRAGS=48 * 8 * 4; // e.g. 48 nodes, 8 frags/node, 4 replicas
-static Uint16 frag_ng_mappings[MAX_FRAGS];
+static Uint32 frag_ng_mappings[MAX_FRAGS];
 static const char* DistTabName= "DistTest";
 static const char* DistTabDKeyCol= "DKey";
 static const char* DistTabPKey2Col= "PKey2";
@@ -55,7 +53,7 @@ setNativePartitioning(Ndb* ndb, NdbDictionary::Table& tab, int when, void* arg)
   }
 
   /* Use rand to choose one of the native partitioning schemes */
-  const Uint32 rType= rand() % 2;
+  const Uint32 rType= rand() % 3;
   Uint32 fragType= -1;
   switch(rType)
   {
@@ -65,6 +63,9 @@ setNativePartitioning(Ndb* ndb, NdbDictionary::Table& tab, int when, void* arg)
   case 1 :
     fragType = NdbDictionary::Object::DistrKeyLin;
     break;
+  case 2:
+    fragType = NdbDictionary::Object::HashMapPartition;
+    break;
   }
 
   ndbout << "Setting fragment type to " << fragType << endl;
@@ -169,7 +170,7 @@ setupUDPartitioning(Ndb* ndb, NdbDictionary::Table& tab)
   {
     frag_ng_mappings[i]= i % numNgs;
   }
-  tab.setFragmentData(frag_ng_mappings, sizeof(uint16) * numPartitions);
+  tab.setFragmentData(frag_ng_mappings, numPartitions);
 
   return 0;
 }
@@ -246,6 +247,10 @@ create_dist_table(Ndb* pNdb,
     {
       setupUDPartitioning(pNdb, tab);
     }
+    else
+    {
+      setNativePartitioning(pNdb, tab, 0, 0);
+    }
     
     NdbDictionary::Column dk;
     dk.setName(DistTabDKeyCol);
@@ -413,7 +418,7 @@ run_create_pk_index(NDBT_Context* ctx, NDBT_Step* step){
   if (pNdb->getDictionary()->createIndex(pIdx) != 0){
     ndbout << "FAILED!" << endl;
     const NdbError err = pNdb->getDictionary()->getNdbError();
-    ERR(err);
+    NDB_ERR(err);
     return NDBT_FAILED;
   }
   
@@ -437,7 +442,7 @@ static int run_create_pk_index_drop(NDBT_Context* ctx, NDBT_Step* step){
   ndbout << "Dropping index " << name.c_str() << " ";
   if (pNdb->getDictionary()->dropIndex(name.c_str(), pTab->getName()) != 0){
     ndbout << "FAILED!" << endl;
-    ERR(pNdb->getDictionary()->getNdbError());
+    NDB_ERR(pNdb->getDictionary()->getNdbError());
     return NDBT_FAILED;
   } else {
     ndbout << "OK!" << endl;
@@ -600,7 +605,7 @@ run_startHint(NDBT_Context* ctx, NDBT_Step* step)
   int result = NDBT_OK;
   for(int i = 0; i<records && result == NDBT_OK; i++)
   {
-    char buffer[8000];
+    char buffer[NDB_MAX_TUPLE_SIZE];
     char* start= buffer + (rand() & 7);
     char* pos= start;
     
@@ -680,9 +685,9 @@ run_startHint_ordered_index(NDBT_Context* ctx, NDBT_Step* step)
   int result = NDBT_OK;
   for(int i = 0; i<records && result == NDBT_OK; i++)
   {
+    char buffer[NDB_MAX_TUPLE_SIZE];
     NdbTransaction* pTrans= NULL;
 
-    char buffer[8000];
     char* start= buffer + (rand() & 7);
     char* pos= start;
     
@@ -1034,6 +1039,7 @@ dist_scan_body(Ndb* pNdb, int records, int parts, PartInfo* partInfo, bool usePr
       //keyParts[0].ptr= &badPartVal;
       
       Ndb::PartitionSpec pSpec;
+      char* tabRow= NULL;
       
       if (userDefined)
       {
@@ -1043,10 +1049,35 @@ dist_scan_body(Ndb* pNdb, int records, int parts, PartInfo* partInfo, bool usePr
       }
       else
       {
-        pSpec.type= Ndb::PartitionSpec::PS_DISTR_KEY_PART_PTR;
-        pSpec.KeyPartPtr.tableKeyParts= keyParts;
-        pSpec.KeyPartPtr.xfrmbuf= NULL;
-        pSpec.KeyPartPtr.xfrmbuflen= 0;
+        /* Can set either using an array of Key parts, or a KeyRecord
+         * structure.  Let's test both
+         */
+        if (rand() % 2)
+        {
+          //ndbout << "Using Key Parts to set range partition info" << endl;
+          pSpec.type= Ndb::PartitionSpec::PS_DISTR_KEY_PART_PTR;
+          pSpec.KeyPartPtr.tableKeyParts= keyParts;
+          pSpec.KeyPartPtr.xfrmbuf= NULL;
+          pSpec.KeyPartPtr.xfrmbuflen= 0;
+        }
+        else
+        {
+          //ndbout << "Using KeyRecord to set range partition info" << endl;
+          
+          /* Setup a row in NdbRecord format with the distkey value set */
+          tabRow= (char*)malloc(NdbDictionary::getRecordRowLength(tabRecord));
+          int& dKeyVal= *((int*) NdbDictionary::getValuePtr(tabRecord,
+                                                            tabRow,
+                                                            tab->getColumn(DistTabDKeyCol)->getAttrId()));
+          dKeyVal= partValue;
+          // dKeyVal= partValue + 1; // Test failue case
+          
+          pSpec.type= Ndb::PartitionSpec::PS_DISTR_KEY_RECORD;
+          pSpec.KeyRecord.keyRecord= tabRecord;
+          pSpec.KeyRecord.keyRow= tabRow;
+          pSpec.KeyRecord.xfrmbuf= 0;
+          pSpec.KeyRecord.xfrmbuflen= 0;
+        }
       }
 
       CHECK(op->setBound(idxRecord,
@@ -1054,6 +1085,11 @@ dist_scan_body(Ndb* pNdb, int records, int parts, PartInfo* partInfo, bool usePr
                          &pSpec,
                          sizeof(pSpec)),
             op);
+
+      if (tabRow)
+        free(tabRow);
+      tabRow= NULL;
+
     }
   }
 
@@ -1175,7 +1211,12 @@ run_dist_test(NDBT_Context* ctx, NDBT_Step* step)
     ->getTable(DistTabName)
     ->getFragmentCount();
   int numDkeyValues= 2*numTabPartitions + (rand() % 6);
-  
+  if (numDkeyValues > records)
+  {
+    // limit number of distributions keys to number of records
+    numDkeyValues = records;
+  }
+
   ndbout << "Table has " << numTabPartitions
          << " physical partitions" << endl;
   ndbout << "Testing with " << numDkeyValues 
@@ -1347,6 +1388,7 @@ NDBT_TESTSUITE_END(testPartitioning);
 
 int main(int argc, const char** argv){
   ndb_init();
+  NDBT_TESTSUITE_INSTANCE(testPartitioning);
   testPartitioning.setCreateTable(false);
   return testPartitioning.execute(argc, argv);
 }
