@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+#include <ndb_global.h>
 #include <NdbRestarts.hpp>
 #include <NDBT.hpp>
 #include <string.h>
@@ -24,7 +25,7 @@
 #include <NdbEnv.h>
 #include <NDBT_Test.hpp>
 
-#define F_ARGS NDBT_Context* ctx, NdbRestarter& _restarter, const NdbRestarts::NdbRestart* _restart
+#define F_ARGS NDBT_Context* ctx, NdbRestarter& _restarter, const NdbRestarts::NdbRestart* _restart, int
 
 int restartRandomNodeGraceful(F_ARGS);
 int restartRandomNodeAbort(F_ARGS);
@@ -242,10 +243,11 @@ const NdbRestarts::NdbRestart* NdbRestarts::getRestart(const char* _name){
 
 int NdbRestarts::executeRestart(NDBT_Context* ctx,
                                 const NdbRestarts::NdbRestart* _restart,
-				unsigned int _timeout){
+				unsigned int _timeout,
+                                int safety){
   // Check that there are enough nodes in the cluster
   // for this test
-  NdbRestarter restarter;
+  NdbRestarter restarter(0, &ctx->m_cluster_connection);
   if (_restart->m_numRequiredNodes > restarter.getNumDbNodes()){
     g_err << "This test requires " << _restart->m_numRequiredNodes << " nodes "
 	  << "there are only "<< restarter.getNumDbNodes() <<" nodes in cluster" 
@@ -258,7 +260,7 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
     return NDBT_FAILED;
   }
   
-  int res = _restart->m_restartFunc(ctx, restarter, _restart);
+  int res = _restart->m_restartFunc(ctx, restarter, _restart, safety);
 
   // Sleep a little waiting for nodes to react to command
   NdbSleep_SecSleep(2);
@@ -280,23 +282,25 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
 
 int NdbRestarts::executeRestart(NDBT_Context* ctx,
                                 int _num,
-				unsigned int _timeout){
+				unsigned int _timeout,
+                                int safety){
   const NdbRestarts::NdbRestart* r = getRestart(_num);
   if (r == NULL)
     return NDBT_FAILED;
 
-  int res = executeRestart(ctx, r, _timeout);
+  int res = executeRestart(ctx, r, _timeout, safety);
   return res;
 }
 
 int NdbRestarts::executeRestart(NDBT_Context* ctx,
                                 const char* _name,
-				unsigned int _timeout){
+				unsigned int _timeout,
+                                int safety){
   const NdbRestarts::NdbRestart* r = getRestart(_name);
   if (r == NULL)
     return NDBT_FAILED;
 
-  int res = executeRestart(ctx, r, _timeout);
+  int res = executeRestart(ctx, r, _timeout, safety);
   return res;
 }
 
@@ -361,7 +365,7 @@ const NdbRestarts::NdbErrorInsert* NdbRestarts::getRandomError(){
 
 int restartRandomNodeGraceful(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
   
@@ -375,7 +379,7 @@ int restartRandomNodeGraceful(F_ARGS){
 
 int restartRandomNodeAbort(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
   
@@ -389,7 +393,7 @@ int restartRandomNodeAbort(F_ARGS){
 
 int restartRandomNodeError(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
   
@@ -415,7 +419,7 @@ int restartMasterNodeError(F_ARGS){
 
 int restartRandomNodeInitial(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
   
@@ -429,7 +433,7 @@ int restartRandomNodeInitial(F_ARGS){
 
 int twoNodeFailure(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int n[2];
   n[0] = _restarter.getDbNodeId(randomId);  
@@ -633,7 +637,6 @@ NFDuringNR_codes[] = {
   7172,
   6000,
   6001,
-  6002,
   7171,
   7130,
   7133,
@@ -659,12 +662,12 @@ NFDuringNR_codes[] = {
   5002
 };
 
-int restartNFDuringNR(F_ARGS){
+int restartNFDuringNR(F_ARGS safety){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int i;
   const int sz = sizeof(NFDuringNR_codes)/sizeof(NFDuringNR_codes[0]);
-  for(i = 0; i<sz; i++){
+  for(i = 0; i<sz && !ctx->closeToTimeout(safety); i++){
     int randomId = myRandom48(_restarter.getNumDbNodes());
     int nodeId = _restarter.getDbNodeId(randomId);
     int error = NFDuringNR_codes[i];
@@ -709,7 +712,7 @@ int restartNFDuringNR(F_ARGS){
   if(NdbEnv_GetEnv("USER", buf, 256) == 0 || strcmp(buf, "ejonore") != 0)
     return NDBT_OK;
   
-  for(i = 0; i<sz && !ctx->isTestStopped(); i++){
+  for(i = 0; i<sz && !ctx->isTestStopped() && !ctx->closeToTimeout(safety);i++){
     const int randomId = myRandom48(_restarter.getNumDbNodes());
     int nodeId = _restarter.getDbNodeId(randomId);
     const int error = NFDuringNR_codes[i];
@@ -787,7 +790,7 @@ NRDuringLCP_NonMaster_codes[] = {
   7018  // Crash in !master when changing state to LCP_TAB_SAVED
 };
 
-int restartNodeDuringLCP(F_ARGS) {
+int restartNodeDuringLCP(F_ARGS safety) {
   int i;
   // Master
   int val = DumpStateOrd::DihMinTimeBetweenLCP;
@@ -795,8 +798,8 @@ int restartNodeDuringLCP(F_ARGS) {
 	"Failed to set LCP to min value"); // Set LCP to min val
   int sz = sizeof(NRDuringLCP_Master_codes)/
            sizeof(NRDuringLCP_Master_codes[0]);
-  for(i = 0; i<sz; i++) {
-
+  for(i = 0; i<sz && !ctx->closeToTimeout(safety); i++)
+  {
     int error = NRDuringLCP_Master_codes[i];
     int masterNodeId = _restarter.getMasterNodeId();
 
@@ -833,7 +836,7 @@ int restartNodeDuringLCP(F_ARGS) {
   // NON-Master
   sz = sizeof(NRDuringLCP_NonMaster_codes)/
        sizeof(NRDuringLCP_NonMaster_codes[0]);
-  for(i = 0; i<sz; i++) {
+  for(i = 0; i<sz && !ctx->closeToTimeout(safety) ; i++) {
 
     int error = NRDuringLCP_NonMaster_codes[i];
     int nodeId = getRandomNodeId(_restarter);
@@ -875,7 +878,7 @@ int restartNodeDuringLCP(F_ARGS) {
 
 int stopOnError(F_ARGS){
 
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
 
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
@@ -901,7 +904,7 @@ int stopOnError(F_ARGS){
 }
 
 int getRandomNodeId(NdbRestarter& _restarter) {
-  myRandom48Init(NdbTick_CurrentMillisecond());
+  myRandom48Init((long)NdbTick_CurrentMillisecond());
   int randomId = myRandom48(_restarter.getNumDbNodes());
   int nodeId = _restarter.getDbNodeId(randomId);
 
