@@ -1,20 +1,20 @@
 #!/usr/bin/perl -w
 
-# Copyright (C) 2005-2008 MySQL AB
-#  All rights reserved. Use is subject to license terms.
+# Copyright (C) 2005-2007 MySQL AB
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
+#   This program is free software; you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation; version 2 of the License.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+#   This program is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+#   You should have received a copy of the GNU General Public License
+#   along with this program; if not, write to the Free Software
+#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+
 
 use strict;
 
@@ -1329,7 +1329,7 @@ sub output
 	printf $f, 'Varsize NULL Bytes/Row';
 	printf $v, $t->vdm_null_bytes->{$_}||0 foreach @{$t->dm_versions};
 	print "\n";
-	printf $f, 'Avg Varside DM/Row';
+	printf $f, 'Avg Varsize DM/Row';
 	printf $v, (exists($t->row_vdm_size->{$_})?
 		    $t->row_vdm_size->{$_}: 0)
 		    foreach @{$r->versions};
@@ -1732,7 +1732,7 @@ ENDHTML
 	    push @r, (exists($t->row_vdm_size->{$_})?
 		      $t->row_vdm_size->{$_}: 0)
 		foreach @{$r->versions};
-	    print $self->tr('Avg Varside DM/Row',@r);
+	    print $self->tr('Avg Varsize DM/Row',@r);
 	}
 	print "</table>\n";
 	print $self->h4("Memory Calculations");
