@@ -1,5 +1,5 @@
-/*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003-2007 MySQL AB
+
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,8 +12,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-*/
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
 
 // Restore
 
@@ -23,13 +22,18 @@
 #include <ndb_global.h>
 #include <NdbOut.hpp>
 #include "../src/kernel/blocks/backup/BackupFormat.hpp"
-#include "../src/ndbapi/NdbDictionaryImpl.hpp"
 #include <NdbApi.hpp>
-#include <util/azlib.h>
+#include <util/ndbzio.h>
+#include <util/UtilBuffer.hpp>
 
 #include <ndb_version.h>
 #include <version.h>
 
+#define NDB_RESTORE_STAGING_SUFFIX "$ST"
+#ifdef ERROR_INSERT
+#define NDB_RESTORE_ERROR_INSERT_SMALL_BUFFER 1
+#endif
+
 enum TableChangesMask
 {
   /**
@@ -89,7 +93,7 @@ struct AttributeDesc {
   friend class TableS;
   friend class RestoreDataIterator;
   friend class RestoreMetaData;
-  friend struct AttributeS;
+  friend class AttributeS;
   Uint32 size; // bits       
   Uint32 arraySize;
   Uint32 attrId;
@@ -100,6 +104,7 @@ struct AttributeDesc {
   AttrConvertFunc convertFunc;
   void *parameter;
   bool truncation_detected;
+  bool staging;
 
 public:
   
@@ -113,7 +118,8 @@ public:
   }
 }; // AttributeDesc
 
-struct AttributeS {
+class AttributeS {
+public:
   AttributeDesc * Desc;
   AttributeData Data;
 };
@@ -196,7 +202,7 @@ public:
   Uint32 getLocalId() const { 
     return m_local_id; 
   }
-  Uint32 getNoOfRecords() const { 
+  Uint64 getNoOfRecords() const { 
     return m_noOfRecords; 
   }
   /*
@@ -292,10 +298,13 @@ public:
   const TableS *getMainTable() const {
     return m_main_table;
   }
+ 
+  Uint32 getMainColumnId() const {
+    return m_main_column_id;
+  }
 
   TableS& operator=(TableS& org) ;
 
-  inline
   bool isSYSTAB_0() const {
     return m_isSYSTAB_0;
   } 
@@ -304,14 +313,18 @@ public:
   bool isBroken() const {
     return m_broken || (m_main_table && m_main_table->isBroken());
   }
-  
+
+  bool m_staging;
+  BaseString m_stagingName;
+  NdbDictionary::Table* m_stagingTable;
+  int m_stagingFlags;
 }; // TableS;
 
 class RestoreLogIterator;
 
 class BackupFile {
 protected:
-  azio_stream m_file;
+  ndbzio_stream m_file;
   char m_path[PATH_MAX];
   char m_fileName[PATH_MAX];
   bool m_hostByteOrder;
@@ -324,12 +337,16 @@ protected:
   void * m_buffer_ptr;
   Uint32 m_buffer_sz;
   Uint32 m_buffer_data_left;
-
+#ifdef ERROR_INSERT
+  unsigned m_error_insert;
+#endif
   Uint64 m_file_size;
   Uint64 m_file_pos;
   
   UtilBuffer m_twiddle_buffer;
 
+  bool  m_is_undolog;
+
   void (* free_data_callback)();
   virtual void reset_buffers() {}
 
@@ -361,6 +378,9 @@ public:
 
   Uint64 get_file_size() const { return m_file_size; }
   Uint64 get_file_pos() const { return m_file_pos; }
+#ifdef ERROR_INSERT
+  void error_insert(unsigned int code); 
+#endif
 
 private:
   void
@@ -401,6 +421,7 @@ public:
   Uint32 getNoOfTables() const { return allTables.size();}
   
   const TableS * operator[](int i) const { return allTables[i];}
+  TableS * operator[](int i) { return allTables[i];}
   TableS * getTable(Uint32 tableId) const;
 
   Uint32 getNoOfObjects() const { return m_objects.size();}
